const bcrypt = require('bcrypt');
const UserServices = require('../Services/User.services');
const OrganizationServices = require('../Services/Organization.services');
const { logger } = require('../logger');

const AuthLogServices = require('../Services/CreateServices')('AuthLog')
const TaskServices = require('../Services/CreateServices')('Task')

exports.login = async (req, res) => {
    try {
        const { platform = "unknown" } = req.body;
        const email = req.body.email.toLowerCase() || "";
        const user_res = await UserServices.getUserByEmail(email);
        if (!user_res.status) return res.status(500).send({ status: false, error: 'Internal Server Error!.' });

        if (user_res.users.length <= 0) {
            return res.status(404).send({ status: false, error: 'Email or Password Invalid!!!' });
        }
        else {
            //get user and match hash password
            let user = user_res.users[0];
            const orgRes = await OrganizationServices.findOne({ _id: user.organization._id });
            user.organization = orgRes.Organizations;

            const isMatch = await bcrypt.compare(req.body.password, user.password);
            if (!isMatch) return res.status(400).json({ status: false, message: "Incorrect Password !" });

            user = user.toJSON();
            delete user.password;

            if (!user.status) {
                return res.status(403).send({ status: false, auth: false, error: 'Invalid User Access. Please Contact Administrator.' });
            }

            const authLogRes = await AuthLogServices.create({ user, organizationId: user.organization._id, platform, type: "login" });

            const clientVisitTaskRes = await TaskServices.find({ query: { 'users.userId': user._id, workType: "CLIENT_VISIT", status: "MY_TASK" }, populate: ['organizationId', 'caseId', 'clientId', 'branchId'] });

            return res.status(200).send({
                status: true,
                body: { user_id: user._id, user: user, token: await UserServices.generateJWTToken(user), clientVisitTask: clientVisitTaskRes.data[0] || null },
            });
        }
    }
    catch (e) {
        logger.error(`🚀 ~ file: Auth.controller.js:37 ~ exports.login= ~ e:", ${e.toString()}`)
        return res.status(500).send({ status: false, error: e ? e : 'Internal Server Error!' });
    }
}

exports.auth = async (req, res) => {
    try {
        const id = req.local._id;
        const { platform = "unknown" } = req.body;
        let user = await UserServices.findById(id);
        const orgRes = await OrganizationServices.findOne({ _id: user.users.organization._id });
        if (!user.status) return res.status(400).send(user);

        let user_data = user.users.toJSON();
        user_data.organization = orgRes.Organizations;
        delete user_data.password;

        const authLogRes = await AuthLogServices.create({ user: user_data, organizationId: user_data.organization._id, platform, type: "auth" });
        const clientVisitTaskRes = await TaskServices.find({ query: { 'users.userId': user_data._id, workType: "CLIENT_VISIT", status: "MY_TASK" }, populate: ['organizationId', 'caseId', 'clientId', 'branchId'] });

        return res.status(200).send({ status: true, auth: true, user: user_data, clientVisitTask: clientVisitTaskRes.data[0] || null });
    }
    catch (e) {
        logger.error(`🚀 ~ file: Auth.controller.js:58 ~ exports.auth= ~ e:", ${e.toString()}`)
        return res.status(500).send({ status: false, auth: false, error: e ? e : 'Internal Server Error!' });
    }
}

exports.logout = async (req, res) => {
    try {
        const id = req.local._id;
        const { platform = "unknown" } = req.body;
        let user = await UserServices.findById(id);
        if (!user.status) return res.status(400).send(user);
        let user_data = user.users.toJSON();
        
        const authLogRes = await AuthLogServices.create({ user: user_data, organizationId: user_data.organization._id, platform, type: "logout" });
        const updateUser = await UserServices.editUser(id, { notificationToken: null });
        return res.status(200).send({ status: true, message:"Succefully logged out." });
    }
    catch (e) {
        logger.error(`🚀 ~ file: Auth.controller.js:58 ~ exports.logout= ~ e:", ${e.toString()}`)
        return res.status(500).send({ status: false, auth: false, error: e ? e : 'Internal Server Error!' });
    }
}