const fs = require('fs');
const path = require('path');
const pdfGenerate = require('../Middleware/pdfGenerate');
const sizeOf = require('image-size');
const sharp = require('sharp');
const EmpanelmentServices = require('../Services/CreateServices')('Empanelment');
const CdnSuccess = require('../Model/CdnSuccess');
const moment = require('moment');
const base64toFile = require('../Utils/Base64ToFile');
const { logger } = require('../logger');
sharp.cache(false);

/**
 * @description This function is used for make PDF from uploaded Images 
 * @param {Array} req.files  from that Array of images details  
 * @param {*} res 
 * @returns Object Result { status: boolean;
    path: string;
    fileName: string; }
id* - { status: false, error: "file not found" }
 */
module.exports.imagestoPdf = async (req, res) => {
    try {
        const id = req.query.id;
        logger.error("in imagestoPDF: ", id)
        if (req.files) {
            // delete req.files.fieldname;
            
            const result = await pdfGenerate.makePDF(req.files, id);

            if (!result.status || result.error) return res.status(500).send(result);
            return res.status(200).send(result);
        } else {
            return res.status(500).send({ status: false, error: "file not found" });
        }
    }
    catch (e) {
        logger.error(`🚀 ~ module.exports.imagestoPdf= ~ e: ${JSON.stringify(e)}`)
        return res.status(500).send({ status: false, error: e.toString() });
    }
};

/**
 * @description This is used for upload One Image  
 * @param {Object} req.file from that get object of uploaded image details 
 * @param {*} res 
 * @returns Object {
                status: true,
                path: req.file.path,
                fileName: req.file.filename,
                size: req.file.size,
                mimeType: req.file.mimetype
            }
 * -  { status: false, error: "file not found" }
 */
module.exports.uploadImage = async (req, res) => {
    try {
        if (req.file) {
            console.log(req.file);
            return res.status(200).send({
                status: true,
                path: req.file.path,
                fileName: req.file.filename,
                size: req.file.size,
                mimeType: req.file.mimetype
            });
        }
        else {
            res.status(500).send({ status: false, error: "file not found" });
        }

    }
    catch (e) {
        logger.error(`🚀 ~ module.exports.uploadImage= ~ e:`, e.toString())
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * @description This is used for upload Multiple Images 
 * @param {Array} req.files get Array of Uploaded images details
 * @param {*} res 
 * @returns Object { status: true, result: array }
 * - { status: false, error: "file not found" }
 */
module.exports.uploadImages = async (req, res) => {

    try {
        var imageArr = [];
        var empDocs = [];
        var empLetter = [];
        var waterMarkArr = [];
        var externalImages = [];
        var stampArr = [];

        const id = req.query.id;
        const orgId = req.query.orgId;

        if (!fs.existsSync(process.env.CDN_PATH + '/cases')) {
            fs.mkdirSync(process.env.CDN_PATH + '/cases', { recursive: true });
        }
        if (!fs.existsSync(process.env.CDN_PATH + '/users')) {
            fs.mkdirSync(process.env.CDN_PATH + '/users', { recursive: true });
        }
        if (!fs.existsSync(process.env.CDN_PATH + '/empanelmentLetters')) {
            fs.mkdirSync(process.env.CDN_PATH + '/empanelmentLetters', { recursive: true });
        }
        if (orgId) {
            if (!fs.existsSync(process.env.CDN_PATH + '/' + orgId)) {
                fs.mkdirSync(process.env.CDN_PATH + '/' + orgId, { recursive: true });
            }
        }
        // ============ Composition Images ===========

        if (req.files.images) {
            if (!fs.existsSync(process.env.CDN_PATH + '/cases/' + id)) {
                fs.mkdirSync(process.env.CDN_PATH + '/cases/' + id, { recursive: true });
            }
            await Promise.all(req.files.images.map(async (file) => {
                let fileName = "compositionImg_" + file.filename;
                let currentPath = process.cwd() + "/" + file.path;
                const destination = path.join(process.env.CDN_PATH, "cases", id, fileName);

                const dimensions = sizeOf(currentPath)
                if (!!dimensions.orientation) {
                    fileName = "compositionImg_r_" + file.filename;
                    const nDestination = path.join(process.env.CDN_PATH, "cases", id, fileName)
                    const { width, height } = await sharp(currentPath).rotate().toFile(nDestination)
                    file.orientation = width < height ? 'portrait' : 'landscape';
                } else {
                    if (dimensions.width < dimensions.height) { file.orientation = "portrait" }
                    else { file.orientation = "landscape" }

                    // call function for move file 
                    await this.moveFile(currentPath, destination);
                }

                file.path = path.join("cases", id, fileName);
                file.filename = fileName;
                file.dimensions = dimensions;
                imageArr.push(file);
            }))

            return res.status(200).send({ status: true, result: imageArr });
        }

        // ============ employeeDocs i.e. for upload employees example : PANcard, Visa etc.======

        if (req.files.employeeDocs) {
            await Promise.all(req.files.employeeDocs.map(async (file) => {

                if (id) {
                    if (!fs.existsSync(process.env.CDN_PATH + '/users/' + id)) {
                        fs.mkdirSync(process.env.CDN_PATH + '/users/' + id);
                    }

                    const fileName = "employeeDocs_" + file.filename;
                    let currentPath = process.cwd() + "/" + file.path;
                    const destination = path.join(process.env.CDN_PATH, "users", id, fileName);

                    // call function for move file 
                    await this.moveFile(currentPath, destination);

                    file.path = path.join("users", id, fileName);
                    empDocs.push({ path: file.path, fileName: fileName });
                }
                else {
                    // console.log("else In utilities Controller");
                    empDocs.push({ path: file.path, fileName: file.filename });
                }

            }))
            // console.log(empDocs, "====== in controller ====== ");
            return res.status(200).send({ status: true, result: empDocs });
        }

        // ======================= Empanelment Letter ========================

        if (req.files.empanelmentLetter) {
            await Promise.all(req.files.empanelmentLetter.map(async (file) => {

                if (id) {

                    if (!fs.existsSync(process.env.CDN_PATH + '/empanelmentLetters/' + id)) {
                        fs.mkdirSync(process.env.CDN_PATH + '/empanelmentLetters/' + id);
                    }
                    const fileName = "empanelmentLetters_" + file.filename;
                    let currentPath = process.cwd() + "/" + file.path;

                    const destination = path.join(process.env.CDN_PATH, "empanelmentLetters", id, fileName);
                    // const destination = "/cdn/empanelmentLetters/" + id + "/" + Date.now() + "_" + file.filename;

                    // call function for move file 
                    await this.moveFile(currentPath, destination);

                    file.path = path.join("empanelmentLetters", id, fileName);

                    empLetter.push({ path: file.path, fileName: fileName });

                } else {
                    empLetter.push({ path: file.path, fileName: file.filename });
                }

            }))
            // console.log(empLetter, "====== in controller ====== ");
            return res.status(200).send({ status: true, result: empLetter });
        }

        // ================== External Building Image ===================

        if (req.files.externalImages) {
            if (!fs.existsSync(process.env.CDN_PATH + '/cases/' + id)) {
                fs.mkdirSync(process.env.CDN_PATH + '/cases/' + id);
            }
            await Promise.all(req.files.externalImages.map(async (file) => {

                let fileName = "externalImg_" + file.filename;
                let currentPath = process.cwd() + "/" + file.path;
                const destination = path.join(process.env.CDN_PATH, "cases", id, fileName);
                const dimensions = sizeOf(currentPath)

                if (!!dimensions.orientation) {
                    fileName = "externalImg_r_" + file.filename;
                    const nDestination = path.join(process.env.CDN_PATH, "cases", id, fileName)
                    const { width, height } = await sharp(currentPath).rotate().withMetadata().toFile(nDestination)
                    file.orientation = width < height ? 'portrait' : 'landscape';
                } else {
                    if (dimensions.width < dimensions.height) { file.orientation = "portrait" }
                    else { file.orientation = "landscape" }

                    // call function for move file 
                    await this.moveFile(currentPath, destination);
                }

                file.path = path.join("cases", id, fileName);
                externalImages.push({ path: file.path, fileName: fileName, dimensions: dimensions, orientation: file.orientation });
            }))
            return res.status(200).send({ status: true, result: externalImages });
        }

        // ================== waterMark Image ===================

        if (req.files.waterMarkImage) {
            if (!fs.existsSync(process.env.CDN_PATH + '/' + orgId + "/reportAssets")) {
                fs.mkdirSync(process.env.CDN_PATH + '/' + orgId + "/reportAssets");
            }
            await Promise.all(req.files.waterMarkImage.map(async (file) => {

                const fileName = "waterMarkImage_" + file.filename;
                let currentPath = process.cwd() + "/" + file.path;
                const destination = path.join(process.env.CDN_PATH, orgId, "reportAssets", fileName);

                // call function for move file 
                await this.moveFile(currentPath, destination);

                file.path = path.join(orgId, "reportAssets", fileName);
                waterMarkArr.push({ path: file.path, fileName: fileName });
            }))
            return res.status(200).send({ status: true, result: waterMarkArr });
        }

        // ================== stamp Image ===================

        if (req.files.stampImage) {
            if (!fs.existsSync(process.env.CDN_PATH + '/' + orgId + "/reportAssets")) {
                fs.mkdirSync(process.env.CDN_PATH + '/' + orgId + "/reportAssets");
            }
            await Promise.all(req.files.stampImage.map(async (file) => {

                const fileName = "stampImage_" + file.filename;
                let currentPath = process.cwd() + "/" + file.path;
                const destination = path.join(process.env.CDN_PATH, orgId, "reportAssets", fileName);

                // call function for move file 
                await this.moveFile(currentPath, destination);

                file.path = path.join(orgId, "reportAssets", fileName);
                stampArr.push({ path: file.path, fileName: fileName });
            }))
            return res.status(200).send({ status: true, result: stampArr });
        }

        return res.status(500).send({ status: false, error: "file not found" });
    }
    catch (e) {
        logger.error(`🚀 ~ module.exports.uploadImages= ~ e:`, e.toString())
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

module.exports.uploadBase64 = async (req, res) => {
    try {
        const id = req.query.id || "TRASH";
        const type = req.query.type || "cases";

        if (req.body.audioBase64) {

            let targetDirectory = path.join(type, id, 'audioNotes');
            const fileName = "audio_" + moment(new Date()).format("dd_mm_yyyy_hh_mm_ss") + ".m4a";
            const { status, filepath, error } = await base64toFile(req.body.audioBase64, fileName, targetDirectory)
            if (status) {
                return res.status(200).send({ status: true, path: filepath, fileName: fileName, date: new Date() });
            }
            else {
                return res.status(500).send({ status: false, error });
            }
        }
        else {
            return res.status(500).send({ status: false, error: "file not found" });
        }
    }
    catch (e) {
        logger.error(`🚀 ~ module.exports.uploadBase64= ~ e:`, e.toString())
        return res.status(500).send({ status: false, error: e.toString() });
    }
}

/**
 * @description This is used for upload  multiple Pdf Files 
 * @param {Array} req.files from that get array of uploded Pdf Details  
 * @param {*} res 
 * @returns Object { status: true, path: string, fileName: string  }
 *- {status : false , error : "file not found" }
 */
module.exports.uploadPDF = async (req, res) => {

    try {
        // console.log(req.files);
        var empDocs = [];
        const id = req.query.id;

        if (req.files.employeeDocs) {
            await Promise.all(req.files.employeeDocs.map(async (file) => {

                if (id) {
                    if (!fs.existsSync(process.env.CDN_PATH + '/users/' + id)) {
                        fs.mkdirSync(process.env.CDN_PATH + '/users/' + id, { recursive: true });
                    }
                    const fileName = "employeeDocs_" + file.filename;
                    let currentPath = process.cwd() + "/" + file.path;

                    const destination = path.join(process.env.CDN_PATH, "users", id, fileName);

                    // call function for move file 
                    await this.moveFile(currentPath, destination);
                    file.path = path.join("users", id, fileName);

                    empDocs.push({ path: file.path, fileName: fileName });
                }
                else {
                    empDocs.push({ path: file.path, fileName: file.filename });
                }

            }))
            return res.status(200).send({ status: true, result: empDocs });
        }

        // ========================== Document, Image related to Case ==========================
        if (req.files.audioNotes) {

            if (id) {
                if (!fs.existsSync(process.env.CDN_PATH + '/cases/' + id + '/audioNotes/')) {
                    fs.mkdirSync(process.env.CDN_PATH + '/cases/' + id + '/audioNotes/', { recursive: true });
                }
                const fileName = "audio_" + req.files.audioNotes[0].filename;
                let currentPath = process.cwd() + "/" + req.files.audioNotes[0].path;

                const destination = path.join(process.env.CDN_PATH, "cases", id, 'audioNotes', fileName);

                // call function for move file 
                await this.moveFile(currentPath, destination);
                const filepath = path.join("cases", id, 'audioNotes', fileName);

                return res.status(200).send({ status: true, path: filepath, fileName: fileName, date: new Date() });
            }
            const filepath = "/" + req.files.audioNotes[0].path;
            return res.status(200).send({ status: true, path: filepath, fileName: req.files.audioNotes[0].filename });
        }

        if (req.files.pdf) {

            if (id) {
                if (!fs.existsSync(process.env.CDN_PATH + '/cases/' + id)) {
                    fs.mkdirSync(process.env.CDN_PATH + '/cases/' + id, { recursive: true });
                }
                const fileName = "document_" + req.files.pdf[0].filename;
                let currentPath = process.cwd() + "/" + req.files.pdf[0].path;

                const destination = path.join(process.env.CDN_PATH, "cases", id, fileName);

                // call function for move file 
                await this.moveFile(currentPath, destination);
                const filepath = path.join("cases", id, fileName);

                return res.status(200).send({ status: true, path: filepath, fileName: fileName });
            }
            const filepath = "/" + req.files.pdf[0].path;
            return res.status(200).send({ status: true, path: filepath, fileName: req.files.pdf[0].filename });
        }
        if (req.files.documents) {

            if (id) {
                if (!fs.existsSync(process.env.CDN_PATH + '/cases/' + id)) {
                    fs.mkdirSync(process.env.CDN_PATH + '/cases/' + id, { recursive: true });
                }
                const fileName = "document_" + req.files.documents[0].filename;
                let currentPath = process.cwd() + "/" + req.files.documents[0].path;

                const destination = path.join(process.env.CDN_PATH, "cases", id, fileName);

                // call function for move file 
                await this.moveFile(currentPath, destination);
                const filepath = path.join("cases", id, fileName);

                return res.status(200).send({ status: true, path: filepath, fileName: fileName });
            }
            const filepath = "/" + req.files.documents[0].path;
            return res.status(200).send({ status: true, path: filepath, fileName: req.files.documents[0].filename });
        }
        if (req.files.empanelmentLetter) {
            if (id) {
                if (!fs.existsSync(process.env.CDN_PATH + '/empanelmentLetters/' + id)) {
                    fs.mkdirSync(process.env.CDN_PATH + '/empanelmentLetters/' + id, { recursive: true });
                }
                const fileName = "empanelmentPdf_" + req.files.empanelmentLetter[0].filename;
                let currentPath = process.cwd() + "/" + req.files.empanelmentLetter[0].path;
                const destination = path.join(process.env.CDN_PATH, "empanelmentLetters", id, fileName);

                // call function for move file 
                await this.moveFile(currentPath, destination);

                const filepath = path.join("empanelmentLetters/", id, "/", fileName);

                return res.status(200).send({ status: true, path: filepath, fileName: fileName });

            }
            else {
                return res.status(200).send({ status: true, fileName: req.files.empanelmentLetter[0].filename, path: req.files.empanelmentLetter[0].path });
            }
        }
        else {
            return res.status(500).send({ status: false, error: "file not found" });
        }

    }
    catch (e) {
        logger.error(`🚀 ~ module.exports.uploadPDF= ~ e:`, e.toString())
        return res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

module.exports.readPermissionsFile = async (req, res) => {

    try {
        var file = fs.readFileSync("./Utils/permissions.json");
        return res.status(200).send({ status: true, permission: file.toString() });

    }
    catch (e) {
        console.log("Error: ", e);
        return res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

module.exports.readDocumentsFile = async (req, res) => {

    try {
        var file = fs.readFileSync("./Utils/documents.json");
        return res.status(200).send({ status: true, documents: file.toString() });
    }
    catch (e) {
        console.log("Error: ", e);
        return res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This function is used for set Expiry status in Empanelment Object 
 * @param {*} req 
 * @param {*} res 
 * @returns Object { status: true }
 */
module.exports.updateExpiryStatus = async (req, res) => {

    try {
        // get all empanelment letter, i.e. not Deleted and not Expired 
        const EmpanelmentResult = await EmpanelmentServices.find({ query: { isDelete: false, expireStatus: { $ne: 'EXPIRED' } }, limit: 1000, populate: ['zoneId', 'valuerId'] });
        if (!EmpanelmentResult.status) return res.status(500).send({ status: false, EmpanelmentResult });

        await Promise.all(EmpanelmentResult.data.map(async (data) => {
            // call function into Empanelment Model 
            await data.isToBeExpired();
        }))
        return res.status(200).send({ status: true });
    }
    catch (e) {
        logger.error(`🚀 ~ module.exports.updateExpiryStatus= ~ e:`, e)
        return res.status(500).send({ status: false, error: e.toString() });
    }
};

/**
 * @description This function used for move file to given destination Path
 * @param {String} currentPath 
 * @param {String} destinationPath 
 */
module.exports.moveFile = async (currentPath, destination) => {

    return await fs.rename(currentPath, destination, async function (err) {
        if (err) {
            const data = {
                sourcePath: currentPath,
                date: new Date(),
                status: false,
                destinationPath: destination,
                error: err
            }
            const result = await CdnSuccess.create(data);
            return { status: false, error: err }
        } else {
            const data = {
                sourcePath: currentPath,
                date: new Date(),
                status: true,
                destinationPath: destination,
            }
            const result = await CdnSuccess.create(data);
            return { status: true, message: "Successfully moved the file!" }
        }
    });
};

/**
 * @description This function used for move file to given destination Path
 * @param {String} currentPath 
 * @param {String} destinationPath 
 */
module.exports.copyFile = async (currentPath, destination) => {

    return await fs.copyFile(currentPath, destination, async function (err) {
        if (err) {
            const data = {
                sourcePath: currentPath,
                date: new Date(),
                status: false,
                destinationPath: destination,
                error: err
            }
            const result = await CdnSuccess.create(data);
            return { status: false, error: err }
        } else {
            const data = {
                sourcePath: currentPath,
                date: new Date(),
                status: true,
                destinationPath: destination,
            }
            const result = await CdnSuccess.create(data);
            return { status: true, message: "Successfully copied the file!" }
        }
    });
};

module.exports.deleteFile = async (destination) => {

    return await fs.unlink(destination, async function (err) {
        if (err) {
            return false;
        } else {
            return true;
        }
    });
};

module.exports.resizeImage = async (req, res) => {
    try {
        const { url, width, height } = req.query;

        if (!url || !width || !height) {
            return res.status(400).json({ error: 'Missing parameters' });
        }
        const resizedImageBuffer = await sharp(process.env.CDN_PATH + url)
            .resize(parseInt(width), parseInt(height))
            .toBuffer();

        res.set('Content-Type', 'image/jpeg'); // You can adjust the content type based on your use case
        res.send(resizedImageBuffer);
    }
    catch (e) {
        logger.error(`🚀 ~ module.exports.resizeImage=async ~ e:` + e.toString())
        res.send({ status: false });
    }
}