const mongoose = require("mongoose");
// const {Point} = require('mongoose-geojson-schema');

const Schema = mongoose.Schema;

const caseSchema = Schema(
    {
        uniqueNo: { type: String },
        inquiryTo: {
            type: Schema.Types.ObjectId,
            ref: "User",
        },
        typeOfReport: {
            value: {
                type: String,
            },
            label: {
                type: String,
            },
        },
        reportInformation: {
            amalgamated: { type: Boolean, default: false },
            proposePurchaser: { type: Boolean, default: false },
            RVDSValues: { type: Boolean, default: true },
            proportionateValue: { type: Boolean, default: false }
        },
        siteVisitContactNo: {
            type: String,
        },
        createCase: {
            type: String,
            default: "false",
        },
        contactPersonNo: {
            type: String,
        },
        contactPersonName: {
            type: String,
        },
        ownerName: {
            type: String,
        },
        proposePurchaserName: {
            type: String,
        },
        finalBuiltupArea: {
            type: String,
        },
        ownerPhoneNo: {
            type: String,
        },
        ownershipShare: {
            type: String,
        },
        email: {
            type: String,
        },
        purchaseValueAsPerDocument: {
            type: String,
        },
        leadFrom: {
            value: {
                type: String,
            },
            label: {
                type: String,
            },
        },
        relationWithOwner: {
            type: String,
        },
        purposeOfValuation: {
            value: {
                type: String,
            },
            label: {
                type: String,
            },
        },
        masterTypeOfPurpose: {
            value: {
                type: String,
            },
            label: {
                type: String,
            },
        },
        toAssess: {
            value: {
                type: String,
            },
            label: {
                type: String,
            },
        },
        sourceOfClient: {
            type: String,
            require: true,
        },
        caseReferredBy: {
            type: String,
        },
        nameOfClient: {
            type: String,
            require: true,
        },
        caseNumber: {
            type: String,
            require: true,
        },
        creditOfficer: {
            _id: {
                type: String,
                // type: Schema.Types.ObjectId,
                // ref: 'User'
            },
            profilePhoto: {
                type: String,
            },
            firstName: {
                type: String,
            },
            lastName: {
                type: String,
            },
            shortName: {
                type: String,
            },
            email: {
                type: String,
            },
            mobileNo: {
                type: String,
            },
            workMobileNo: {
                type: String,
            },
            type: {
                type: String,
            },
            branch: {
                _id: {
                    type: String,
                },
            },
            workWithDocumentMaker: {
                type: String,
            },
        },
        alternateContactNo: {
            type: String,
        },
        fees: {
            type: {
                type: String,
                require: true,
            },
            propertyValue: {
                type: String,
                require: true,
            },
            discussWithPerson: {
                type: String,
            },
            expectedFees: {
                type: Number,
            },
            reason: {
                type: String,
            },
            quotedAmount: {
                type: String,
                require: true,
            },
        },
        organization: {
            _id: {
                type: String,
                // ref: 'Organization'
            },
            name: {
                type: String,
            },
            shortName: {
                type: String,
            },
            email: {
                type: String,
            },
            address: {
                type: String,
            },
            city: {
                type: String,
            },
            phone: {
                type: String,
            },
            profilePhoto: {
                type: String,
            }
        },
        type: {
            type: String,
            default: "LEAD",
        },
        leadType: {
            type: {},
            default: { label: "Lead", value: "LEAD" }
        },
        client: {
            _id: {
                type: String,
                // type: Schema.Types.ObjectId,
                // ref: 'Clients'
            },
            type: {
                type: String,
                // required: true
            },
            name: {
                type: String,
                // required: true
            },
            displayName: {
                type: String,
            },
            initial: {
                type: String,
            },
            clientKey: {
                type: String,
            },
            remark: {
                type: String,
            },
            isdelete: {
                type: Boolean,
            },
            status: {
                type: Boolean,
            },
        },
        branch: {
            _id: {
                type: String,
                // type: Schema.Types.ObjectId,
                // ref: 'Branch'
            },
            type: {
                type: String,
            },
            branchCode: {
                type: String,
            },
            name: {
                type: String,
            },
            email: {
                type: String,
            },
            mobileNo: {
                type: String,
            },
            alternateMobileNo: {
                type: String,
            },
            address: {
                type: String,
            },
            sendStructuralCertificate: {
                type: String,
            },
            sendStructuralReport: {
                type: String,
            },
            ledgerName: {
                type: String,
            },
            fax: {
                type: String,
            },
            description: {
                type: String,
            },
        },
        assetsClass: {
            // 1) Land & Building / Immovable Property  2) Plant & Machinery  3) Securities or Financial Assets

            label: {
                type: String,
            },
            value: {
                type: String,
            },
        },
        propertyType: {
            label: {
                type: String,
                default: "Residential"
            },
            value: {
                type: String,
                default: "Residential"
            },
        },
        zone: {
            _id: {
                type: String,
            },
            zoneName: {
                type: String,
            },
        },
        staff: {
            pickupPerson: {
                _id: {
                    type: String,
                    // type: Schema.Types.ObjectId,
                    // ref: 'User'
                },
                profilePhoto: {
                    type: String,
                },
                firstName: {
                    type: String,
                },
                lastName: {
                    type: String,
                },
                shortName: {
                    type: String,
                },
                email: {
                    type: String,
                },
                mobileNo: {
                    type: String,
                },
                role: {
                    type: String,
                },
                type: {
                    type: String,
                },
                branch: {
                    _id: {
                        type: String,
                        // type: Schema.Types.ObjectId,
                        // ref: 'Organization'
                    },
                },
                workWithDocumentMaker: {
                    type: String,
                },
            },
            deliveryPerson: {
                _id: {
                    type: String,
                    // type: Schema.Types.ObjectId,
                    // ref: 'User'
                },
                profilePhoto: {
                    type: String,
                },
                firstName: {
                    type: String,
                },
                lastName: {
                    type: String,
                },
                shortName: {
                    type: String,
                },
                email: {
                    type: String,
                },
                mobileNo: {
                    type: String,
                },
                role: {
                    type: String,
                },
                type: {
                    type: String,
                },
                workWithDocumentMaker: {
                    type: String,
                },
            },
            siteEngineer: {
                _id: {
                    type: String,
                    // type: Schema.Types.ObjectId,
                    // ref: 'User'
                },
                profilePhoto: {
                    type: String,
                },
                firstName: {
                    type: String,
                },
                lastName: {
                    type: String,
                },
                shortName: {
                    type: String,
                },
                email: {
                    type: String,
                },
                designation: {
                    type: String,
                },
                mobileNo: {
                    type: String,
                },
                role: {
                    type: String,
                },
                type: {
                    type: String,
                },
                workWithDocumentMaker: {
                    type: String,
                },
            },
            rateVerifier: {
                _id: {
                    type: String,
                    // type: Schema.Types.ObjectId,
                    // ref: 'User'
                },
                profilePhoto: {
                    type: String,
                },
                firstName: {
                    type: String,
                },
                lastName: {
                    type: String,
                },
                shortName: {
                    type: String,
                },
                email: {
                    type: String,
                },
                designation: {
                    type: String,
                },
                mobileNo: {
                    type: String,
                },
                role: {
                    type: String,
                },
                type: {
                    type: String,
                },
                workWithDocumentMaker: {
                    type: String,
                },
            },
            maker: {
                _id: {
                    type: String,
                    // type: Schema.Types.ObjectId,
                    // ref: 'User'
                },
                profilePhoto: {
                    type: String,
                },
                firstName: {
                    type: String,
                },
                lastName: {
                    type: String,
                },
                shortName: {
                    type: String,
                },
                email: {
                    type: String,
                },
                designation: {
                    type: String,
                },
                mobileNo: {
                    type: String,
                },
                role: {
                    type: String,
                },
                type: {
                    type: String,
                },
                workWithDocumentMaker: {
                    type: String,
                },
            },
            checker: {
                _id: {
                    type: String,
                    // type: Schema.Types.ObjectId,
                    // ref: 'User'
                },
                profilePhoto: {
                    type: String,
                },
                firstName: {
                    type: String,
                },
                lastName: {
                    type: String,
                },
                shortName: {
                    type: String,
                },
                email: {
                    type: String,
                },
                designation: {
                    type: String,
                },
                mobileNo: {
                    type: String,
                },
                role: {
                    type: String,
                },
                type: {
                    type: String,
                },
                workWithDocumentMaker: {
                    type: String,
                },
            },
            valuers: [
                {
                    _id: {
                        type: String,
                        // type: Schema.Types.ObjectId,
                        // ref: 'User'
                    },
                    firstName: {
                        type: String,
                    },
                    fatherName: {
                        type: String,
                    },
                    PANNo: {
                        type: String,
                    },
                    lastName: {
                        type: String,
                    },
                    profilePhoto: {
                        type: String,
                    },
                    shortName: {
                        type: String,
                    },
                    email: {
                        type: String,
                    },
                    designation: {
                        type: String,
                    },
                    mobileNo: {
                        type: String,
                    },
                    role: {
                        type: String,
                    },
                    type: {
                        type: String,
                    },
                    registrationNo: [
                        {
                            title: {
                                value: {
                                    type: String,
                                },
                                label: {
                                    type: String,
                                },
                            },
                            number: {
                                type: String,
                            },
                        },
                    ],
                    workWithDocumentMaker: {
                        type: String,
                    },
                    valuersRegistrationNumber: [
                        {
                            type: Object,
                        },
                    ],
                },
            ],
        },
        owners: [
            {
                _id: false,
                id: String,
                name: String,
                phone: String,
                email: String,
                aadharNo: String,
                panNo: String,
                ownershipShare: String,
                address: String,
                relation: String,
                alternateMobileNo: String,
            },
        ],
        proposePurchasers: [
            {
                _id: false,
                id: String,
                name: String,
                phone: String,
                email: String,
                aadharNo: String,
                panNo: String,
                ownershipShare: String,
                address: String,
                relation: String,
                alternateMobileNo: String,
            },
        ],
        fullAddress: {
            type: String,
        },
        appAddress: {
            type: String,
        },
        addressForMap: {
            type: String,
        },
        clientType: {
            value: {
                type: String,
            },
            label: {
                type: String,
            },
        },
        remark: {
            type: String,
        },
        location: {
            latitude: String,
            longitude: String,
        },
        geo: {
            type: {
                type: String,
                enum: ['Point'],
                default: "Point",
                required: true
            },
            coordinates: {
                type: [Number],
                default: [0, 0]
            }
        },
        siteInspection: [
            {
                areaType: {
                    value: String,
                    label: String,
                },
                area: String,
                providedBy: {
                    value: String,
                    label: String,
                },
                providedByName: String,
                providedByContactNo: String,
            },
        ],
        siteVisitType: { type: String, default: "FULL" },
        siteVisit: {
            schedule: [
                {
                    _id: false,
                    dateTime: String,
                    remark: String,
                },
            ],
            siteVisitStatus: {
                type: Boolean,
                default: false,
            },
            visitedDate: [
                {
                    _id: false,
                    remark: { type: String },
                    status: { type: String },
                    dateTime: { type: String },
                },
            ],
        },
        marketAnalysisAndRemark: {
            areaType: {}, // builtUp, Carpet, Saleable
            rateMin: { type: String },
            rateMax: { type: String },
            lumpsumValue: { type: String },
            remark: { type: String },
            audioNotes: [{}],
        },
        siteVisitDate: {
            type: String,
        },
        scheduleSiteVisitDate: {
            type: String,
        },
        composition: [
            {
                _id: false,
                id: String,
                name: String,
                postName: String,
                measurementType: {
                    value: {
                        type: String,
                    },
                    label: {
                        type: String,
                    },
                },
                length: String,
                width: String,
                length1: String,
                width1: String,
                length2: String,
                width2: String,
                totalArea: String,
                finalArea: String,
                images: [
                    {
                        _id: false,
                        id: String,
                        name: String,
                        path: String,
                        orientation: String,
                        uploadedBy: { type: String },
                        updatedAt: String,
                    },
                ],
                remark: String,
                percentageBeforeTotal: String,
                extraAreas: [
                    {
                        name: String,
                        measurementType: {
                            value: {
                                type: String,
                            },
                            label: {
                                type: String,
                            },
                        },
                        extraAreaType: String,
                        length: String,
                        width: String,
                        length1: String,
                        width1: String,
                        length2: String,
                        width2: String,
                        totalArea: String,
                    },
                ],
            },
        ],
        otherComposition: [
            {
                _id: false,
                id: String,
                toBeAdded: {
                    value: {
                        type: String,
                    },
                    label: {
                        type: String,
                    },
                },
                name: {
                    value: {
                        type: String,
                    },
                    label: {
                        type: String,
                    },
                },
                measurementType: {
                    value: {
                        type: String,
                    },
                    label: {
                        type: String,
                    },
                },
                length: String,
                width: String,
                length1: String,
                width1: String,
                length2: String,
                width2: String,
                totalArea: String,
                finalArea: String,
                // otherComposition: String,
                images: [
                    {
                        _id: false,

                        id: String,
                        name: String,
                        path: String,
                        orientation: String,
                        uploadedBy: { type: String },
                        updatedAt: String,
                    },
                ],
                remark: String,
                percentageBeforeTotal: String,
                percentage: String,
                extraAreas: [
                    {
                        name: String,
                        measurementType: {
                            value: {
                                type: String,
                            },
                            label: {
                                type: String,
                            },
                        },
                        extraAreaType: String,
                        length: String,
                        width: String,
                        length1: String,
                        width1: String,
                        length2: String,
                        width2: String,
                        totalArea: String,
                    },
                ],
            },
        ],
        otherCompositionCarpetTotal: {
            type: String,
        },
        otherCompositionBuiltUpTotal: {
            type: String,
        },
        otherCompositionSaleableTotal: {
            type: String,
        },
        compositionFlatType: {
            value: {
                type: String,
            },
            label: {
                type: String,
            },
        },
        externalReport: [
            {
                _id: false,
                path: String,
                fileName: String,
                createdAt: String,
                uploadedBy: { type: Schema.Types.ObjectId, ref: "User" },
            },
        ],
        externalReportDetails: {
            area: String,
            rate: String,
            yearOfConstruction: String,
            compositionOfBuilding: String,
            finalAreaType: { type: String, default: "Built_Up" },
            finalMeasurementType: { type: String, default: "Sq. Ft." },
            underConstruction: { type: Boolean, default: false },
        },
        extendOfSiteRemark: { type: String, },
        siteMeasurementRemark: { type: String },
        documents: [
            {
                _id: false,
                id: String,
                path: String,
                fileName: String,
                documentType: {
                    value: {
                        type: String,
                    },
                    label: {
                        type: String,
                    },
                },
                inputs: {
                    type: Object,
                },
                displayStatus: Boolean,
                isHardCopy: Boolean,
                createdAt: String,
                verified: String,
                uploadedBy: { type: Schema.Types.ObjectId, ref: "User" },
            },
        ],
        isUrgent: {
            type: String,
        },
        isExternal: {
            type: String,
        },
        isExternalReport: {
            type: Boolean,
            default: false
        },
        verifiedDocument: {
            value: Boolean,
            remark: String,
            dateTime: String,
        },
        documentPerArea: [
            {
                _id: false,
                id: String,
                selectAreaTypeDocument: String,
                typeOfDocument: {
                    value: {
                        type: String,
                    },
                    label: {
                        type: String,
                    },
                },
                docMeasurementType: {
                    value: {
                        type: String,
                    },
                    label: {
                        type: String,
                    },
                },
                docFinalBuiltUpSquareMeter: {
                    type: String,
                },
                squareMeterArea: String,
                totalAreaDocument: String,
                docBuiltupPercentage: String,
                docFinalBuiltUp: String,
                docSaleablePercentage: String,
                docFinalSaleable: String,
                docFinalSaleableSquareMeter: String,
            },
        ],
        caseHistory: [
            {
                _id: false,
                id: String,
                title: String,
                description: String,
                datetime: String,
            },
        ],
        isdelete: {
            type: Boolean,
        },
        address: {
            state: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            country: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            village: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            floorNo: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            buildingNo: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            district: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            city: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            tahasilNo: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            taluka: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            plotNo: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            buildingName: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            selectArea: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            street1: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            street2: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            landmark: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            wing: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            projectName: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            oldSurveyNumber: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            newSurveyNumber: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            hissaNo: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            CTSNo: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            blockNo: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            selectNo: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            municipalityWardNo: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            khataNo: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            mouza: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            SRO: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            policeStation: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            GUTNo: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            postalCode: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
            suburb: {
                name: String,
                value: String,
                key: String,
                status: { type: Boolean, default: false },
                label: Boolean,
            },
        },
        images: [
            {
                id: String,
                name: String,
                path: String,
                uploadedBy: { type: Schema.Types.ObjectId, ref: "User" },
                updatedAt: String,
            },
        ],
        flatSpecifications: {
            roof: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            flooring: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            doors: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            windows: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            fittings: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            finishing: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
        },
        extraDetailOfProperty: {
            type: String,
        },
        briefDescOfProperty: {
            type: String,
        },
        general: {
            ownership: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            // planningAreaList: {
            //     value: {
            //         type: String
            //     },
            //     label: {
            //         type: String
            //     }
            // },
            populationClassList: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            populationGroupList: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            //underTownPlanningList: {
            // type: String
            // },
            extraDetailOfProperty: {
                type: String,
            },
            townplanApprovedByList: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            whetherCoveredUnderAnyState: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            asPerActualAgreement: {
                north: {
                    type: String,
                },
                west: {
                    type: String,
                },
                south: {
                    type: String,
                },
                east: {
                    type: String,
                },
            },
            asPerDocumentAgreement: {
                north: {
                    type: String,
                },
                west: {
                    type: String,
                },
                south: {
                    type: String,
                },
                east: {
                    type: String,
                },
            },
            unitBoundaries: {
                title: { type: String },
                asPerActual: { north: { type: String }, west: { type: String }, south: { type: String }, east: { type: String } },
                asPerDocument: { north: { type: String }, west: { type: String }, south: { type: String }, east: { type: String } },
                isDisplay: { type: Boolean, default: false },
            },
            dimensions: {
                asPerActual: {
                    north: { type: String },
                    west: { type: String },
                    south: { type: String },
                    east: { type: String },
                },
                asPerDocument: {
                    north: { type: String },
                    west: { type: String },
                    south: { type: String },
                    east: { type: String },
                }
            },
            dateOfIssusApprovePlan: { type: String },
            approvePlanVerified: { type: String },
            empanelledOfApprovePlan: { type: String },
            unauthorizedConstruction: { type: String },
            extraDetailOfBuilding: { type: String },
            whichTheLocalityIsServed: {
                buses: {
                    title: { type: String, default: "Buses" },
                    status: { type: Boolean }
                },
                taxies: {
                    title: { type: String, default: "Taxies" },
                    status: { type: Boolean }
                },
                auto: {
                    title: { type: String, default: "Auto" },
                    status: { type: Boolean }
                },
                privateCars: {
                    title: { type: String, default: "Private Cars" },
                    status: { type: Boolean }
                }
            },
            classOfSanitary: {},
            compoundWall: { type: String }
        },
        landDetails: {},
        shopDetails: {},
        landAndBuildingDetails: {
            landDetails: {},
            buildingDetails: {},
            otherDetails: {},
            areaDetails: {},
            valuationDetails: {},
            summaryDetails: {},
            guidelineRateDetails: {},
            specificationOfConstruction: {},
            briefDescriptionOfLand: { type: String },
            briefDescriptionOfStructure: { type: String },
            briefDescriptionOfProperty: { type: String },
            possibilityOfFrequentFlooding: { type: String },
            feasibilityTOTheCivicAmenities: { type: String },
            levelOfLandWithTopographicalCondition: { type: String },
            shapeOfLand: { type: String },
            shapeOfLandTypeOfUseToWhichItCanBePut: { type: String },
            anyUsageRestriction: { type: String },
            CornerPlotOrIntermittentPlot: { type: String },
            roadFacilities: { type: String },
            typeOfRoadAvailableAtPresent: { type: String },
            widthOfRoad: { type: String },
            isItLandOrLockLand: { type: String },
            briefDescriptionOfAsPerDocumentOrPlan: { type: String },
            conditionOfTheBuildExterior: {},
            conditionOfTheBuildInterior: {},
            actualBuildingBoundaries: {},
            documentsBuildingBoundaries: {},
            advantagesOfTheSite: { type: String },
            powerSupplyAvailableInTheSite: {},
        },
        reraId: { type: String },
        apfDetails: {
            buildings: [{}],
            compositions: [{}]
        },
        compositeRate: {
            remark: {
                type: String,
            },
            depreciationPercentage: {
                type: String,
            },
            salvageValue: {
                type: String,
            },
            depreciatedBuildingRate: {
                type: String,
            },
            buildingServicesDepreciatedRate: {
                type: String,
            },
            depreciatedCostOfConstruction: {
                type: String,
            },
            depreciationRatioOfBuilding: {
                type: String,
            },
        },
        indicatorArr: [
            {
                id: String,
                property: {
                    type: String,
                },
                rateType: {
                    type: String,
                },
                squareMeter: {
                    type: String,
                },
                area: {
                    type: String,
                },
                areaType: {
                    type: String,
                },
                builtUpExpectedRate: {
                    type: String,
                },
                builtUpPercentage: {
                    type: String,
                },
                saleablePercentage: {
                    type: String,
                },
                floor: {
                    type: String,
                },
                source: {
                    type: String,
                    // label: { type: String },
                    // value: { type: String }
                },
                expectedPrice: {
                    type: String,
                },
                expectedRate: {
                    type: String,
                },
                image: {
                    type: String,
                },
                printInReport: {
                    type: Boolean,
                    default: true
                },
                carpetArea: {
                    area: { type: String },
                    squareMeterArea: { type: String },
                    expectedRate: { type: String },
                    percentage: { type: String, }
                },
                builtUpArea: {
                    area: { type: String },
                    squareMeterArea: { type: String },
                    expectedRate: { type: String },
                    percentage: { type: String, }
                },
                saleableArea: {
                    area: { type: String },
                    squareMeterArea: { type: String },
                    expectedRate: { type: String },
                    percentage: { type: String, }
                },
                "plotArea": {
                    "squareFeet": { type: String, },
                    "squareMeter": { type: String, }
                },
                "structureArea": {
                    "squareFeet": { type: String, },
                    "squareMeter": { type: String, }
                },
                "costOfArea": {
                    "squareFeet": { type: String, },
                    "squareMeter": { type: String, }
                },
                "costOfStructure": { type: String, },
                "netValueOfLand": { type: String, },
                "netRateForLand": {
                    "squareFeet": { type: String, },
                    "squareMeter": { type: String, }
                }
            },
        ],
        igrRate: {
            propertyName: { type: String },
            rate: { type: String },
            landRate: { type: String },
            igrImage: { type: String },
            squareMeterGuideLineRate: { type: String },
            squareFeetGuideLineRate: { type: String },
        },
        igrFinalRate: {
            depreciationPercentage: { type: String },
            rate: { type: String },
            squareMeterGuideLineRate: { type: String },
            squareFeetGuideLineRate: { type: String },
            floorRatePercentage: { type: String, default: '0.00' }
        },
        otherRateDetails: {},
        outgoingDetail: {
            assessmentNo: {
                type: String,
            },
            taxPaidHolderName: {
                type: String,
            },
            tenantName: {
                type: String,
            },
            taxAmount: {
                type: String,
            },
            electricityServiceConnectionNo: {
                type: String,
            },
            meterCardHolderName: {
                type: String,
            },
            maintenanceOfFlat: {
                type: String,
            },
            maintenanceOfGala: {
                type: String,
            },
            // saleDeedExecutedName: {
            //     type: String
            // },
            undividedAreaOfLandAsPerSaleDeed: {
                type: String,
            },
            floorSpaceIndex: {
                type: String,
            },
            areaStatusList: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            propertyTypeOfUseList: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            propertyOccupiedByList: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            monthlyRent: {
                type: String,
                default: "",
            },
        },
        flatRate: {
            type: String,
        },
        buildingServicesRate: {
            type: String,
        },
        landOtherRate: {
            type: String,
        },
        propertyValuation: {
            type: String,
        },
        valuationAreaType: {
            type: String,
        },
        sourceValuation: {
            type: String,
        },
        marketability: {
            marketabilityList: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            anyFavouringForExtraPotentialValue: {
                type: String,
            },
            negativeFactorsOfAffectMarketValue: {
                type: String,
            },
        },
        siteMeasurement: {
            roof: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            partitions: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            waterTank: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            pavementLaid: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            carParking: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            constructionWorkStatus: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            asthetikLook: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            dateOfConstruction: {
                date: String,
                month: String,
                year: String,
                fullDate: String,
            },
            totalLifeBuilding: {
                type: String,
            },
            ageOfBuilding: {
                type: String,
            },
            estimatedAge: {
                type: String,
            },
            propertyTypeList: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            planningArea: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            typeOfStructure: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            typeOfFoundation: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            qualityOfConstruction: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            materialUsedList: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            buildingSpecification: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            structuralSafety: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            protAgainstNatDis: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            waterSourceList: {
                type: String,
            },
            sewerageSystem: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            walls: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            appearanceOfTheBuilding: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            maintenanceOfTheBuilding: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            maintenanceIssues: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            visibleDamageInBuilding: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            systemOfAirConditioning: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            provisionOfFirefighting: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            noOfBasement: {
                type: String,
            },
            groundStilt: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            noOfPodium: {
                type: String,
            },
            noOfAmenityFloors: {
                type: String,
            },
            noOfFloors: {
                type: String,
            },
            flatFloorNo: {
                type: String,
            },
            noOfLifts: {
                type: String,
            },
            noOfLiftsInEachWings: {
                type: String,
            },
            noOfDwellingUnitsInBuilding: {
                type: String,
            },
            isSiteVisitAllowed: {
                type: Boolean,
                default: true,
            },
            yearOfConstructionAsPer: {
                value: {
                    type: String,
                },
                label: {
                    type: String,
                },
            },
            constructionRemarks: { type: String }
        },
        isSiteMeasurementAllowed: {
            type: Boolean,
            default: true
        },
        finalPlinthArea: {
            areaAsPer: { type: String },
            area: { type: String, default: "0.00" },
            measurementType: String,
            areaType: { type: String }
        },
        constructionDetails: {
            // podiumWork: {
            //     type: String
            // },
            // podiumPercentage: {
            //     type: String
            // },
            rccFoundationWork: {
                type: String,
            },
            rccFoundationPercentage: {
                type: String,
            },
            rccPlinth: {
                type: String,
            },
            rccPlinthPercentage: {
                type: String,
            },
            // basementWork: {
            //     type: String
            // },
            // basementPercentage: {
            //     type: String
            // },
            // groundStiltFloors: {
            //     type: String
            // },
            // groundStiltFloorsPercentage: {
            //     type: String
            // },
            // amenityFloorsWork: {
            //     type: String
            // },
            // amenityFloorsPercentage: {
            //     type: String
            // },
            floorsWork: {
                type: String,
            },
            floorsPercentage: {
                type: String,
            },
            internalBrickWork: {
                type: String,
            },
            internalBrickWorkPercentage: {
                type: String,
            },
            externalBrickWork: {
                type: String,
            },
            externalBrickWorkPercentage: {
                type: String,
            },
            internalPlasteringWork: {
                type: String,
            },
            internalPlasteringPercentage: {
                type: String,
            },
            externalPlasteringWork: {
                type: String,
            },
            externalPlasteringPercentage: {
                type: String,
            },
            flooringTillingKitchenWork: {
                type: String,
            },
            flooringTillingKitchenPercentage: {
                type: String,
            },
            internalPaintingWork: {
                type: String,
            },
            internalPaintingPercentage: {
                type: String,
            },
            externalPaintingWork: {
                type: String,
            },
            externalPaintingPercentage: {
                type: String,
            },
            electrificationPlumbingSanitaryWork: {
                type: String,
            },
            electrificationPlumbingSanitaryPercentage: {
                type: String,
            },
            liftInstallation: {
                type: String,
            },
            liftPercentage: {
                type: String,
            },
            // openAreaDevFinalFinishing: {
            //     type: Boolean,
            //     default: false
            // },
            openAreaDevFinalFinishingPercentage: {
                type: String,
            },
            workComplete: {
                type: String,
            },
            passageStaircaseAndLobbyPercentage: {
                type: String,
            },
            doorWork: {
                type: String,
            },
            doorPercentage: {
                type: String,
            },
            windowWork: {
                type: String,
            },
            windowPercentage: {
                type: String,
            },
        },
        asthetikLook: {
            value: {
                type: String,
            },
            label: {
                type: String,
            },
        },
        routeMap: {
            locationMapImg: [
                {
                    _id: false,
                    id: String,
                    name: String,
                    image: String,
                    uploadedBy: { type: Schema.Types.ObjectId, ref: "User" },
                    updatedAt: String,
                },
            ],
            routeMapDetail: {
                routeMapImg: { type: String, default: "" },
                routeLat: { type: String, default: "" },
                routeLng: { type: String, default: "" },
                routeStation: {},
                stationName: {
                    type: String, default: "",
                },
                routeDistance: { type: String, default: "" },
            },
            finalLatLng: {
                // index: true,
                type: {
                    type: String,
                    enum: ["Point"],
                    // default: 'Point'
                    // enum: ['Point'], // 'location.type' must be 'Point'
                },
                coordinates: {
                    type: [Number],
                    // index: "2d", // for createIndex // for near by location
                },
            },
            showLocation: {
                latlng: Boolean,
                finallatlng: Boolean,
            },
        },
        valuation: {
            wardrobesRate: {
                type: String,
            },
            totalwardrobesRate: {
                type: String,
            },
            wardrobesQty: {
                type: String,
            },
            showcasesQty: {
                type: String,
            },
            showcasesRate: {
                type: String,
            },
            totalshowcasesRate: {
                type: String,
            },
            kitchenArrangementsQty: {
                type: String,
            },
            kitchenArrangementsRate: {
                type: String,
            },
            totalkitchenArrangementsRate: {
                type: String,
            },
            superfineFinishQty: {
                type: String,
            },
            superfineFinishRate: {
                type: String,
            },
            totalsuperfineFinishRate: {
                type: String,
            },
            interiorDecorationsQty: {
                type: String,
            },
            interiorDecorationsRate: {
                type: String,
            },
            totalinteriorDecorationsRate: {
                type: String,
            },
            electricityDepositsQty: {
                type: String,
            },
            electricityDepositsRate: {
                type: String,
            },
            totalelectricityDepositsRate: {
                type: String,
            },
            extraCollapsibleGatesQty: {
                type: String,
            },
            extraCollapsibleGatesRate: {
                type: String,
            },
            totalextraCollapsibleGatesRate: {
                type: String,
            },
            potentialValueQty: {
                type: String,
            },
            potentialValueRate: {
                type: String,
            },
            totalpotentialValueRate: {
                type: String,
            },
            othersQty: {
                type: String,
            },
            othersRate: {
                type: String,
            },
            totalothersRate: {
                type: String,
            },
            finaltotal: {
                type: String,
            },
            finalValue: {
                type: String,
            },
            bookValue: {
                type: String,
            },
            realizableValuePercentage: {
                type: String,
            },
            distressValuePercentage: {
                type: String,
            },
            bookValueDocument: {
                type: Object,
            },
            purchaseValueDocument: {
                type: Object,
            },
            fairMarketValue: {
                costOfConstruction: { type: String, default: "" },
                depreciationPercentage: { type: String, default: "" },
                depreciatedCostOfConstruction: { type: String, default: "" },
                totalFairMarketValue: { type: String, default: "" }
            }
        },
        carpetTotal: {
            type: String,
        },
        otherCarpetTotal: {
            type: String,
        },
        toBeAdded: {
            type: String,
        },
        finalArea: {
            type: String,
        },
        extraAreaTotalCount: {
            type: String,
        },
        builtupPercentage: {
            type: String,
        },
        salablePercentage: {
            type: String,
        },
        finalCarpetArea: {
            type: String,
        },
        finalSalableArea: {
            type: String,
        },
        finalValuationArea: {
            type: String,
            default: "0",
        },
        finalAreaType: {
            type: String,
        },
        finalMeasurementType: {
            type: String,
        },
        finalAsPerArea: {
            type: String,
        },
        finalCarpetConvertToBuiltUpPer: {
            type: String,
        },
        finalBuiltUpPercentage: {
            type: String,
        },
        finalSaleablePercentage: {
            type: String,
        },
        finalValuationAreaString: {
            type: String,
        },
        finalCarpetArea: {
            type: String,
        },
        reportGeneratedDate: {
            type: String,
        },
        outward: {
            number: String,
            date: String,
            datetime: { type: Date },
            status: {
                type: Boolean,
                default: false
            },
            remark: {
                type: String,
                default: "SYSTEM_GENERATED",
            },
            dayCount: { type: String },
            monthCount: { type: String },
            generatedBy: {
                type: Schema.Types.ObjectId,
                ref: 'User'
            }
        },
        empanelmentNo: { type: [String] },
        rateVerify: {
            dateTime: String,
            status: {
                type: Boolean,
                default: false,
            },
            user: {
                _id: String,
                firstName: String,
                lastName: String,
            },
        },
        underConstruction: {
            rccFootingFoundation: {
                label: {
                    type: String,
                    default: "RCC Footing/Foundation",
                },
                floorsCompleted: {
                    type: String,
                    default: "",
                },
                percentage: {
                    type: String,
                    default: "0.00",
                },
                isHideFloor: {
                    type: Boolean,
                    default: false,
                },
                status: {
                    type: Boolean,
                    default: true,
                },
            },
            rccPlinth: {
                label: {
                    type: String,
                    default: "RCC Plinth",
                },
                floorsCompleted: {
                    type: String,
                    default: "",
                },
                percentage: {
                    type: String,
                    default: "0.00",
                },
                isHideFloor: {
                    type: Boolean,
                    default: false,
                },
                status: {
                    type: Boolean,
                    default: false,
                },
            },
            fullBuildingRcc: {
                label: {
                    type: String,
                    default: "Full Building Rcc",
                },
                floorsCompleted: {
                    type: String,
                    default: "",
                },
                percentage: {
                    type: String,
                    default: "0.00",
                },
                isHideFloor: {
                    type: Boolean,
                    default: false,
                },
                status: {
                    type: Boolean,
                    default: false,
                },
            },
            internalBrickWork: {
                label: {
                    type: String,
                    default: "Internal Brick Work",
                },
                floorsCompleted: {
                    type: String,
                    default: "",
                },
                percentage: {
                    type: String,
                    default: "0.00",
                },
                isHideFloor: {
                    type: Boolean,
                    default: false,
                },
                status: {
                    type: Boolean,
                    default: false,
                },
            },
            externalBrickWork: {
                label: {
                    type: String,
                    default: "External Brick Work",
                },
                floorsCompleted: {
                    type: String,
                    default: "",
                },
                percentage: {
                    type: String,
                    default: "0.00",
                },
                isHideFloor: {
                    type: Boolean,
                    default: false,
                },
                status: {
                    type: Boolean,
                    default: false,
                },
            },
            internalPlastering: {
                label: {
                    type: String,
                    default: "Internal Plastering",
                },
                floorsCompleted: {
                    type: String,
                    default: "",
                },
                percentage: {
                    type: String,
                    default: "0.00",
                },
                isHideFloor: {
                    type: Boolean,
                    default: false,
                },
                status: {
                    type: Boolean,
                    default: false,
                },
            },
            externalPlastering: {
                label: {
                    type: String,
                    default: "External Plastering",
                },
                floorsCompleted: {
                    type: String,
                    default: "",
                },
                percentage: {
                    type: String,
                    default: "0.00",
                },
                isHideFloor: {
                    type: Boolean,
                    default: false,
                },
                status: {
                    type: Boolean,
                    default: false,
                },
            },
            door: {
                label: {
                    type: String,
                    default: "Door",
                },
                floorsCompleted: {
                    type: String,
                    default: "",
                },
                percentage: {
                    type: String,
                    default: "0.00",
                },
                isHideFloor: {
                    type: Boolean,
                    default: false,
                },
                status: {
                    type: Boolean,
                    default: false,
                },
            },
            window: {
                label: {
                    type: String,
                    default: "Window",
                },
                floorsCompleted: {
                    type: String,
                    default: "",
                },
                percentage: {
                    type: String,
                    default: "0.00",
                },
                isHideFloor: {
                    type: Boolean,
                    default: false,
                },
                status: {
                    type: Boolean,
                    default: false,
                },
            },
            flooringTillingKitchenPlatform: {
                label: {
                    type: String,
                    default: "Flooring, Tilling, Kitchen Platform",
                },
                floorsCompleted: {
                    type: String,
                    default: "",
                },
                percentage: {
                    type: String,
                    default: "0.00",
                },
                isHideFloor: {
                    type: Boolean,
                    default: false,
                },
                status: {
                    type: Boolean,
                    default: false,
                },
            },
            internalPainting: {
                label: {
                    type: String,
                    default: "Internal Painting",
                },
                floorsCompleted: {
                    type: String,
                    default: "",
                },
                percentage: {
                    type: String,
                    default: "0.00",
                },
                isHideFloor: {
                    type: Boolean,
                    default: false,
                },
                status: {
                    type: Boolean,
                    default: false,
                },
            },
            externalPainting: {
                label: {
                    type: String,
                    default: "External Painting",
                },
                floorsCompleted: {
                    type: String,
                    default: "",
                },
                percentage: {
                    type: String,
                    default: "0.00",
                },
                isHideFloor: {
                    type: Boolean,
                    default: false,
                },
                status: {
                    type: Boolean,
                    default: false,
                },
            },
            electrificationPlumbingSanitaryInstallation: {
                label: {
                    type: String,
                    default: "Electrification, Plumbing, Sanitary Installation",
                },
                floorsCompleted: {
                    type: String,
                    default: "",
                },
                percentage: {
                    type: String,
                    default: "0.00",
                },
                isHideFloor: {
                    type: Boolean,
                    default: false,
                },
                status: {
                    type: Boolean,
                    default: false,
                },
            },
            liftInstallation: {
                label: {
                    type: String,
                    default: "Lift Installation",
                },
                floorsCompleted: {
                    type: String,
                    default: "",
                },
                percentage: {
                    type: String,
                    default: "0.00",
                },
                isHideFloor: {
                    type: Boolean,
                    default: false,
                },
                status: {
                    type: Boolean,
                    default: false,
                },
            },
            passageStaircaseLobbyDevelopment: {
                label: {
                    type: String,
                    default: "Passage, Staircase & Lobby Development",
                },
                floorsCompleted: {
                    type: String,
                    default: "",
                },
                percentage: {
                    type: String,
                    default: "0.00",
                },
                isHideFloor: {
                    type: Boolean,
                    default: true,
                },
                status: {
                    type: Boolean,
                    default: false,
                },
            },
            externalDevelopmentFinalFinishing: {
                label: {
                    type: String,
                    default: "External Development & Final Finishing",
                },
                floorsCompleted: {
                    type: String,
                    default: "",
                },
                percentage: {
                    type: String,
                    default: "0.00",
                },
                isHideFloor: {
                    type: Boolean,
                    default: true,
                },
                status: {
                    type: Boolean,
                    default: false,
                },
            },
            workCompleted: {
                label: {
                    type: String,
                    default: "Work Completed (%)",
                },
                floorsCompleted: {
                    type: String,
                    default: "",
                },
                percentage: {
                    type: String,
                    default: "0.00",
                },
                isHideFloor: {
                    type: Boolean,
                    default: true,
                },
                status: {
                    type: Boolean,
                    default: true,
                },
            },
        },
        guidelineDetails: {
            finalGuidelineValue: {
                type: String,
                default: "0",
            },
            selectedBuildUpValue: {
                type: String,
                default: "0",
            },
            selectedBuildUpMeasurementType: {
                type: String,
                default: "",
            },
        },
        externalImages: [
            {
                name: String,
                path: String,
                orientation: String,
                type: { type: String, default: "EXTERNAL_IMAGES" },
                status: {
                    type: Boolean,
                    default: false,
                },
            },
        ],
        reportImages: {
            frontPageImages: [
                {
                    name: String,
                    path: String,
                    orientation: String,
                    status: {
                        type: Boolean,
                        default: false,
                    },
                },
            ],
            selfieWithSiteVisitImages: [
                {
                    name: String,
                    path: String,
                    orientation: String,
                    status: {
                        type: Boolean,
                        default: false,
                    },
                },
            ],
            externalPhotos: [
                {
                    name: String,
                    path: String,
                    orientation: String,
                    status: {
                        type: Boolean,
                        default: false,
                    },
                },
            ],
            parkingImages: [
                {
                    name: String,
                    path: String,
                    orientation: String,
                    status: {
                        type: Boolean,
                        default: false,
                    },
                },
            ],
            societyNameBoardImage: [
                {
                    name: String,
                    path: String,
                    orientation: String,
                    status: {
                        type: Boolean,
                        default: false,
                    },
                },
            ],
            compositionImages: [
                {
                    name: String,
                    path: String,
                    orientation: String,
                    status: {
                        type: Boolean,
                        default: false,
                    },
                },
            ],
        },
        externalDocuments: [{
            _id: false,
            id: String,
            originalname: String,
            path: String,
            filename: String,
            createdAt: String,
            size: String,
            documentType: {},
            uploadedBy: { type: Schema.Types.ObjectId, ref: "User" },
        }],
        processingDocuments: [{
            _id: false,
            id: String,
            originalname: String,
            path: String,
            filename: String,
            createdAt: String,
            size: String,
            documentType: {},
            uploadedBy: { type: Schema.Types.ObjectId, ref: "User" },
        }],
        indicatorRange: {
            carpetRateRange: {
                min: { type: String, default: '0.00' }, max: { type: String, default: "0.00" }
            },
            builtUpRateRange: {
                min: { type: String, default: "0.00" }, max: { type: String, default: "0.00" }
            },
            saleableRateRange: {
                min: { type: String, default: "0.00" }, max: { type: String, default: "0.00" }
            }
        },
        detailsOfValuation: {
            presentValueOfFlat: {
                title: { type: String, default: "Present value of the flat (incl. car parking, if provided)" },
                quantity: { type: String, default: '0.00' },
                ratePerUnit: { type: String, default: "0.00" },
                estimatedValue: { type: String, default: "0.00" },
                status: { type: Boolean, default: true },
                isEditable: { type: Boolean, default: false }
            },
            wardrobes: {
                title: { type: String, default: "Wardrobes" },
                quantity: { type: String, default: '0.00' },
                ratePerUnit: { type: String, default: "0.00" },
                estimatedValue: { type: String, default: "0.00" },
                status: { type: Boolean, default: true },
                isEditable: { type: Boolean, default: true }
            },
            showcases: {
                title: { type: String, default: "Showcases" },
                quantity: { type: String, default: '0.00' },
                ratePerUnit: { type: String, default: "0.00" },
                estimatedValue: { type: String, default: "0.00" },
                status: { type: Boolean, default: true },
                isEditable: { type: Boolean, default: true }
            },
            kitchenArrangements: {
                title: { type: String, default: "Kitchen arrangements" },
                quantity: { type: String, default: '0.00' },
                ratePerUnit: { type: String, default: "0.00" },
                estimatedValue: { type: String, default: "0.00" },
                status: { type: Boolean, default: true },
                isEditable: { type: Boolean, default: true }
            },
            superfineFinish: {
                title: { type: String, default: "Superfine finish" },
                quantity: { type: String, default: '0.00' },
                ratePerUnit: { type: String, default: "0.00" },
                estimatedValue: { type: String, default: "0.00" },
                status: { type: Boolean, default: true },
                isEditable: { type: Boolean, default: true }
            },
            interiorDecorations: {
                title: { type: String, default: "Interior Decorations" },
                quantity: { type: String, default: '0.00' },
                ratePerUnit: { type: String, default: "0.00" },
                estimatedValue: { type: String, default: "0.00" },
                status: { type: Boolean, default: true },
                isEditable: { type: Boolean, default: true }
            },
            electricityDeposits: {
                title: { type: String, default: "Electricity deposits / electrical fittings, etc" },
                quantity: { type: String, default: '0.00' },
                ratePerUnit: { type: String, default: "0.00" },
                estimatedValue: { type: String, default: "0.00" },
                status: { type: Boolean, default: true },
                isEditable: { type: Boolean, default: true }
            },
            extraCollapsibleGates: {
                title: { type: String, default: "Extra collapsible gates / grill works etc" },
                quantity: { type: String, default: '0.00' },
                ratePerUnit: { type: String, default: "0.00" },
                estimatedValue: { type: String, default: "0.00" },
                status: { type: Boolean, default: true },
                isEditable: { type: Boolean, default: true }
            },
            potentialValue: {
                title: { type: String, default: "Potential value, if any" },
                quantity: { type: String, default: '0.00' },
                ratePerUnit: { type: String, default: "0.00" },
                estimatedValue: { type: String, default: "0.00" },
                status: { type: Boolean, default: true },
                isEditable: { type: Boolean, default: true }
            },
            others: {
                title: { type: String, default: "Others / Car Parking" },
                quantity: { type: String, default: '0.00' },
                ratePerUnit: { type: String, default: "0.00" },
                estimatedValue: { type: String, default: "0.00" },
                status: { type: Boolean, default: true },
                isEditable: { type: Boolean, default: true }
            },
            totalValues: {
                title: { type: String, default: "Total" },
                quantity: { type: String, default: '' },
                ratePerUnit: { type: String, default: "" },
                estimatedValue: { type: String, default: "0.00" },
                status: { type: Boolean, default: true },
                isEditable: { type: Boolean, default: false }
            },
        },
        isAdvanceInvoice: { type: Boolean, default: false },
        gstNo: { type: String },
        status: {
            type: Boolean,
            default: true,
        },
        caseStatus: {
            type: String,
            default: "LEAD", // "LEAD" ,CASE, documenrVerified, documentUnverified, Draft, remainingSiteVisit, rateFinalized onHold, sitevisitCompleted, draftCompleted, verified, reportGenerated, holdForPayment, signedReportAttach, sentToDelivery, Deilverd, paymentPending, closed, cancled
        },
        paymentStatus: {
            type: String,
            default: "NA",
            enum: ["PENDING", "PART_RECEIVED", "RECEIVED", "NA"]
        },
        reason: {
            type: String,
        },
        createdBy: {
            type: Schema.Types.ObjectId,
            ref: "User",
        },
        updatedBy: {
            type: Schema.Types.ObjectId,
            ref: "User",
        },
        invoiceIds: [{
            type: Schema.Types.ObjectId,
            ref: 'nInvoice',
        }],
        paymentIds: [{
            type: Schema.Types.ObjectId,
            ref: 'Payment',
        }],
        assignmentType: {},
        taskStatus: {
            SITE_INSPECTION: { type: Boolean, default: false },         // TRUE
            RATE_VERIFY: { type: Boolean, default: false },             // TRUE
            MAKER: { type: Boolean, default: false },                   // TRUE
            CHECKER: { type: Boolean, default: false },                 // 3
            PREPARE_INVOICE: { type: Boolean, default: false },         // 4
            PRINT: { type: Boolean, default: false },                   // 5
            SIGN: { type: Boolean, default: false },                    // 6 auto assign to valuer
            SIGNED_REPORT_UPLOAD: { type: Boolean, default: false },    // 7
            DELIVERY: { type: Boolean, default: false },                // 8
            PAYMENT_COLLECT: { type: Boolean, default: false },
            COMPLETED: { type: Boolean, default: false }
        },
        leadRemarks: [
            {
                id: { type: Number },
                note: { type: String },
                createdBy: {
                    type: Schema.Types.ObjectId,
                    ref: "User"
                },
                // createdBy: {
                //     _id: {
                //         type: Schema.Types.ObjectId,
                //         ref: "User"
                //     },
                //     firstName: { type: String },
                //     lastName: { type: String },
                //     role: { type: String },
                //     profilePhoto: { type: String },
                // },
                createdAt: { type: Date },
            }
        ],
        audioRemarks: [{
            id: { type: Number },
            path: { type: String },
            fileName: { type: String },
            createdBy: {
                type: Schema.Types.ObjectId,
                ref: "User"
            },
            createdAt: { type: Date },
        }],
        remarks: [
            {
                note: { type: String },
                workType: { type: String, default: "CASE" },
                display: { type: Boolean, default: true },
                status: { type: Boolean, default: true },
                taskId: {
                    type: Schema.Types.ObjectId,
                    ref: "Task",
                },
                createdBy: {
                    _id: {
                        type: Schema.Types.ObjectId,
                        ref: "User"
                    },
                    firstName: { type: String },
                    lastName: { type: String },
                    role: { type: String },
                    profilePhoto: { type: String },
                },
                createdAt: { type: Date },
            }
        ],
        refCaseId: {
            type: Schema.Types.ObjectId,
            ref: 'Case'
        }
    },
    { timestamps: true }
);

// This is used for change rateverified status
caseSchema.methods.rateVerifier = async function (user) {
    this.rateVerify.status = false;
    this.rateVerify.dateTime = new Date();
    this.rateVerify.user = user;
    await this.save();
    return;
};

/**
 * @description This function is used for add document.
 * @param {array} documents
 * This function is call from caseController
 */
// caseSchema.methods.addDoc = async function (documents, userID) {
//     const doc = await Promise.all(documents.map(async (document) => {
//         if (!document.id) {
//             const doc_id = Math.floor(100000 + Math.random() * 900000);
//             document.id = doc_id;
//             document.uploadedBy = userID;
//             document.createdAt = new Date();
//         }
//         // if (document.documentType.value == "Electricity Bill") {
//         //     var num = document.inputs.number.value;
//         //     var name = document.inputs.to.value;
//         //     // console.log(document.inputs[num]);
//         //     // datas.outgoingDetail = datas.outgoingDetail ? datas.outgoingDetail : {}
//         //     this.outgoingDetail.electricityServiceConnectionNo = document.inputs[num];
//         //     this.outgoingDetail.meterCardHolderName = document.inputs[name];
//         // }
//         // else {
//         //     this.outgoingDetail.electricityServiceConnectionNo = "Details not available";
//         //     this.outgoingDetail.meterCardHolderName = "Details not available";
//         // }
//         // if (document.documentType.value == "Property Tax") {
//         //     var num = document.inputs.number.value;
//         //     var name = document.inputs.to.value;
//         //     // console.log(document.inputs[num]);
//         //     // console.log(document.inputs[name]);
//         //     // console.log(document.inputs.amount);
//         //     // datas.outgoingDetail = datas.outgoingDetail ? datas.outgoingDetail : {}
//         //     this.outgoingDetail.assessmentNo = document.inputs[num];
//         //     this.outgoingDetail.taxPaidHolderName = document.inputs[name];
//         //     this.outgoingDetail.taxAmount = document.inputs.amount;
//         // } else {
//         //     this.outgoingDetail.assessmentNo = "Details not available";
//         //     this.outgoingDetail.taxPaidHolderName = "Details not available";
//         //     this.outgoingDetail.taxAmount = "Details not available";
//         // }
//         return document;

//     }))
//     this.documents = doc;
//     await this.save();
//     return doc;
// }
module.exports = mongoose.model("Case", caseSchema);
