const mongoose = require('mongoose');
const { workTypeArr } = require('../Types/Task');

const Schema = mongoose.Schema;

const taskSchema = Schema({
    workType: {
        type: String,
        default: "CASE",
        enum: workTypeArr
    },
    organizationId: {
        type: Schema.Types.ObjectId,
        ref: 'Organization',
    },
    clientType: {},
    clientId: {
        type: Schema.Types.ObjectId,
        ref: 'Clients',
    },
    branchId: {
        type: Schema.Types.ObjectId,
        ref: 'Branch',
    },
    caseId: {
        type: Schema.Types.ObjectId,
        ref: 'Case',
    },
    clientName: { type: String },
    users: [
        {
            _id: false,
            userId: {
                type: Schema.Types.ObjectId,
                ref: 'User',
            },
            firstName: { type: String },
            lastName: { type: String },
            role: { type: String },
        }
    ],
    timeLog: [
        {
            time: { type: Date, default: Date.now },
            geo: {
                type: {
                    type: String,
                    enum: ['Point'],
                    default: "Point",
                    required: true
                },
                coordinates: {
                    type: [Number],
                    // index: { type: '2dsphere', spares: true },
                    default: [0, 0]
                }
            },
            type: { type: String, default: "START" },
        }
    ],
    duration: { type: Number, default: 0 },
    deadline: { type: Date, default: Date.now },
    priority: { type: Number, enum: [0, 1, 2], default: 0 },
    remark: { type: String },
    images: [{}],
    audioNotes: [{}],
    remarks: [{}],
    review: [
        {
            _id: false,
            _id: { type: String, default: Date.now },
            toTask: {
                _id: { type: Schema.Types.ObjectId, ref: "Task" },
                workType: { type: String },
                users: [{}]
            },
            type: { type: String }, // RAISED, RAISED_FOR, HOLD
            status: { type: String, default: "OPEN" }, // OPEN, COMPLETED
            reason: { type: String, },
            date: { type: Date },
            completedAt: { type: Date },
        }
    ],
    geo: {
        type: {
            type: String,
            enum: ['Point'],
            default: "Point",
            required: true
        },
        coordinates: {
            type: [Number],
            // index: { type: '2dsphere', spares: true },
            default: [0, 0]
        }
    },
    assignedBy: {
        type: Schema.Types.ObjectId,
        ref: 'User',
    },
    pickedAt: { type: Date },
    completedAt: { type: Date },
    status: {
        type: String,
        default: "OPEN",
        enum: ["OPEN", "MY_TASK", "REVIEW", "CHECKED_IN", "COMPLETED", "CANCELLED"]
    }
}, { timestamps: true });

taskSchema.index({ "geo.coordinates": "2dsphere" });

module.exports = mongoose.model('Task', taskSchema);