const Attendance = require('../Model/Attendance');
/**
 * @description This is used for get list of All Attendances
 * @param {*} query 
 * @returns Object { status: true, Attendances: Array }
 * - { status: false, error: string }
 */
module.exports.getAllAttendance = async (query = {}, limit, startIndex) => {

    return Attendance.find(query).skip(startIndex).limit(limit)
        .then((Attendances) => {
            return { status: true, Attendances: Attendances };
        }).catch((e) => {
            return { status: false, error: e };
        });

};
/**
 * @description This is used for get count of documents 
 * @param {*} query 
 * @returns Object { status: true, Attendances: Object }
 *  { status: false, error: String }
 */
 module.exports.getCount = async (query) => {
    // console.log(" Query => ", query)
    return Attendance.countDocuments(query).then((Attendances) => {
        return { status: true, Attendances: Attendances };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get one Attendance By Id
 * @param {string} id 
 * @returns  Object { status: true, Attendances: Object }
 * - { status: false, error: string }
 */
module.exports.getAttendance = async (id) => {

    return Attendance.findById(id).then((Attendance) => {
        return { status: true, Attendances: Attendance };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get one Attendance
 * @param {*} query 
 * @returns Object { status: true, Attendances: Object }
 * - { status: false, error: string }
 */
module.exports.findAttendance = async (query = {}) => {

    // console.log(query);
    return Attendance.findOne(query).then((Attendance) => {
        return { status: true, Attendances: Attendance };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for create Attendance
 * @param {Object} AttendanceData
 * @returns Object { status: true, Attendances: Object }
 * - { status: false, error: string } 
 */
module.exports.createAttendance = async (AttendanceData) => {

    return Attendance.create(AttendanceData)
        .then((result) => {
            // console.log("create Attendance Service")
            return { status: true, Attendances: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};
/**
 * @description This is used for update Attendance
 * @param {String} id 
 * @param {Object} AttendanceData 
 * @returns Object { status: true, Attendances: Object }
 * - { status: false, error: string } 
 */
module.exports.updateAttendance = async (id, AttendanceData) => {

    return Attendance.findOneAndUpdate({ _id: id }, AttendanceData,{new: true})
    
        .then((result) => {
            return { status: true, Attendances: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};