const CreateContact = require('../Model/CreateContact');
/**
 * @description This is used for get list of all CreateContacts
 * @param {*} query 
 * @returns Object { status: true, CreateContacts: Array }
 * - { status: false, error: string }
 */
module.exports.getAllCreateContacts = async (query = {}, limit, startIndex) => {

    return CreateContact.find(query).skip(startIndex).limit(limit).sort({ createdAt: -1 })
        .then((CreateContacts) => {
            return { status: true, CreateContacts: CreateContacts };
        }).catch((e) => {
            return { status: false, error: e };
        });

};
/**
 * @description This is used for get count of documents 
 * @param {*} query 
 * @returns Object { status: true, CreateContacts: Object }
 *  { status: false, error: String }
 */
module.exports.getCount = async (query) => {
    // console.log(" Query => ", query)
    return CreateContact.countDocuments(query).then((CreateContacts) => {
        return { status: true, CreateContacts: CreateContacts };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get One CreateContact By Id 
 * @param {String} id 
 * @returns Object { status: true, CreateContacts: object }
 * - { status: false, error: string }
 */
module.exports.getCreateContact = async (id) => {

    return CreateContact.findById(id).then((CreateContact) => {
        return { status: true, CreateContacts: CreateContact };
    }).catch((e) => {
        return { status: false, error: e };
    });

};

// module.exports.findCreateContact = async (id) => {

//     return CreateContact.findById(id).then((CreateContact) => {
//         return { status: true, CreateContacts: CreateContact };
//     }).catch((e) => {
//         return { status: false, error: e };
//     });

// };
/**
 * @description This is used for create CreateContact  
 * @param {Object} CreateContactData 
 * @returns Object { status: true, CreateContacts: object }
 * - { status: false, error: string } 
 */
module.exports.createCreateContact = async (CreateContactData) => {

    return CreateContact.create(CreateContactData)
        .then((result) => {
            // console.log("create CreateContact Service")
            return { status: true, CreateContacts: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};
/**
 * @description This is used for update CreateContact 
 * @param {Strin} id 
 * @param {Object} CreateContactData 
 * @returns Object { status: true, CreateContacts: object }
 * - { status: false, error: string } 
 */
module.exports.updateCreateContact = async (id, CreateContactData) => {

    return CreateContact.findOneAndUpdate({ _id: id }, CreateContactData, { new: true })
        .then((result) => {
            return { status: true, CreateContacts: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};