const DynamicDropDown = require('../Model/DynamicDropDown');
/**
 * @description This is used for get list of all DynamicDropDowns
 * @param {*} query 
 * @returns Object { status: true, dropdowns: Array }
 * - { status: false, error: string }
 */
module.exports.getAllDynamicDropDowns = async (query = {}, startIndex, limit) => {

    return DynamicDropDown.find(query).collation({ 'locale': 'en' }).sort({ name: 1 }).skip(startIndex).limit(limit)
        .then((DynamicDropDowns) => {
            // console.log("query :", query);
            return { status: true, dropdowns: DynamicDropDowns };
        }).catch((e) => {
            console.log(e);
            return { status: false, error: e };
        });
};
/**
 * @description This is used for get count of documents 
 * @param {*} query 
 * @returns Object { status: true, dropdowns: Object }
 *  { status: false, error: String }
 */
module.exports.getCount = async (query) => {
    // console.log(" Query => ", query)
    return DynamicDropDown.countDocuments(query)
        .then((DynamicDropDowns) => {
            return { status: true, dropdowns: DynamicDropDowns };
        }).catch((e) => {
            return { status: false, error: e };
        });

};
/**
 * @description This is used for get One DynamicDropDown
 * @param {*} query 
 * @returns Object { status: true, dropdowns: Object }
 * - { status: false, error: string }
 */
module.exports.getDynamicDropDown = async (query = {}) => {

    return DynamicDropDown.findOne(query).then((DynamicDropDowns) => {
        // console.log("query :", query);
        return { status: true, dropdowns: DynamicDropDowns };
    }).catch((e) => {
        console.log(e);
        return { status: false, error: e };
    });
};
/**
 * @description This is used for Create DynamicDropDown
 * @param {Object} dropdown_data 
 * @returns Object { status: true, dropdowns: Object }
 * - { status: false, error: string }
 */
module.exports.createDynamicDropDown = async (dropdown_data) => {
    return DynamicDropDown.create(dropdown_data)
        .then((DynamicDropDowns) => {
            // console.log("query :", query);
            return { status: true, dropdowns: DynamicDropDowns };
        }).catch((e) => {
            console.log(e);
            return { status: false, error: e };
        });
};
/**
 * @description This is used for update DynamicDropDown
 * @param {String} name 
 * @param {Object} dropdown_data 
 * @returns Object { status: true, dropdowns: Object }
 * - { status: false, error: string }
 */
module.exports.editDynamicDropDown = async (name, dropdown_data) => {
    return DynamicDropDown.findOneAndUpdate({ name: name }, dropdown_data, { new: true })
        .then((DynamicDropDowns) => {
            // console.log("query :", query);
            return { status: true, dropdowns: DynamicDropDowns };
        }).catch((e) => {
            console.log(e);
            return { status: false, error: e };
        });
};

module.exports.deleteDynamicDropDown = async (req, res) => {
    // return res.status(200).send({ message: "Hi There Its  Admin" });
};

module.exports.editAllDynamicDropDown = async (query, dropdown_data) => {
    return DynamicDropDown.updateMany(query, dropdown_data, { new: true })
        .then((DynamicDropDowns) => {
            console.log("query :", query, dropdown_data);
            return { status: true, dropdowns: DynamicDropDowns };
        }).catch((e) => {
            console.log(e);
            return { status: false, error: e };
        });
};