const Holiday = require('../Model/Holiday');
/**
 * @description This is used for get list of All holidays
 * @param {*} query 
 * @returns Object { status: true, holidays: Array }
 * - { status: false, error: string }
 */
module.exports.getAllHoliday = async (query = {}, limit, startIndex) => {

    return Holiday.find(query).skip(startIndex).limit(limit)
        .then((Holidays) => {
            return { status: true, holidays: Holidays };
        }).catch((e) => {
            return { status: false, error: e };
        });

};
/**
 * @description This is used for get count of documents 
 * @param {*} query 
 * @returns Object { status: true, holidays: Object }
 *  { status: false, error: String }
 */
 module.exports.getCount = async (query) => {
    // console.log(" Query => ", query)
    return Holiday.countDocuments(query).then((Holidays) => {
        return { status: true, holidays: Holidays };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get one Holiday By Id
 * @param {string} id 
 * @returns  Object { status: true, holidays: Object }
 * - { status: false, error: string }
 */
module.exports.getHoliday = async (id) => {

    return Holiday.findById(id).then((Holiday) => {
        return { status: true, holidays: Holiday };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get one Holiday
 * @param {*} query 
 * @returns Object { status: true, holidays: Object }
 * - { status: false, error: string }
 */
module.exports.findHoliday = async (query = {}) => {

    // console.log(query);
    return Holiday.findOne(query).then((Holiday) => {
        return { status: true, holidays: Holiday };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for create Holiday
 * @param {Object} HolidayData
 * @returns Object { status: true, holidays: Object }
 * - { status: false, error: string } 
 */
module.exports.createHoliday = async (HolidayData) => {

    return Holiday.create(HolidayData)
        .then((result) => {
            // console.log("create Holiday Service")
            return { status: true, holidays: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};
/**
 * @description This is used for update Holiday
 * @param {String} id 
 * @param {Object} HolidayData 
 * @returns Object { status: true, holidays: Object }
 * - { status: false, error: string } 
 */
module.exports.updateHoliday = async (id, HolidayData) => {

    return Holiday.findOneAndUpdate({ _id: id }, HolidayData,{new: true})
    
        .then((result) => {
            return { status: true, holidays: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};