const Package = require('../Model/Package');
/**
 * @description This is used for get list of All Packages 
 * @param {*} query 
 * @returns Object { status: true, packages: Array }
 * - { status: false, error: String }
 */
module.exports.getAllPackages = async (query = {}) => {

    return Package.find(query).then((Packages) => {
        return { status: true, packages: Packages };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get One Package By Id
 * @param {String} id 
 * @returns Object { status: true, packages: Object }
 * - { status: false, error: String }
 */
module.exports.getPackage = async (id) => {

    return Package.findById(id).then((Package) => {
        return { status: true, packages: Package };
    }).catch((e) => {
        return { status: false, error: e };
    });

};

// module.exports.findPackage = async (id) => {

//     return Package.findById(id).then((Package) => {
//         return { status: true, Packages: Package };
//     }).catch((e) => {
//         return { status: false, error: e };
//     });

// };
/**
 * @description This is used for create Package
 * @param {Object} PackageData 
 * @returns Object { status: true, packages: Object }
 * - { status: false, error: String }
 */
module.exports.createPackage = async (PackageData) => {

    return Package.create(PackageData)
        .then((result) => {
            // console.log("create Package Service")
            return { status: true, packages: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};
/**
 * @description This is used for update Package
 * @param {String} id 
 * @param {Object} PackageData 
 * @returns Object { status: true, packages: Object }
 * - { status: false, error: String }
 */
module.exports.updatePackage = async (id, PackageData) => {
    
    return Package.findOneAndUpdate({ _id: id }, PackageData)
        .then((result) => {
            return { status: true, packages: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};