const fs = require('fs');


/*
* Use : This function is used to convert request permissions to binary bits with permission.
*
* Parameters : 
*           - permissions : permissions object from request ie. {"case":{"access":{"value":true},"create":{"value":true},"delete":{"value":true}}
*
* Response : 
*           - Success :
*               - {status: true, result:{case:0101, roles: 0110}}
*           - Permission Mismatch: 
*               - {status: true, result: "fail"}
*           - Error :
*               - {status: false}
*
* Calling From :
*           - RoleController
*/
PermissionToBinary = async (permissions)=>{

    try{
        const perJson = await fs.readFileSync('./Utils/permissions.json');
        
        var permissionJson = {};
        var bitArr = {};
        var permissionBit = {};

        // ===== get data from permission.json file and set it to permission format ie.{case:{access:1|2|4|8}} =====
        await Promise.all(
            JSON.parse(perJson).map(async (json)=>{
        
                var tempJson = {};
                await Promise.all(

                    json.permission.map(async (permsn)=>{
                        tempJson[permsn.label] = permsn.value;
                    })
                )
                permissionJson[json.title] = tempJson;
            })
        );
        
        // ===== match request permission with permission.json file and set bits on permission position ie.{case:{'1':0, '2':1, '4': 0, '8': 1}} =====
        await Promise.all(
            Object.keys(permissions).map(async(mainPermission)=>{
                
                const JSONtitle = mainPermission;
                if(mainPermission in permissionJson){
                    
                    await Promise.all(
                        Object.keys(permissions[mainPermission]).map(async(subPermission)=>{
                            
                            if(subPermission in permissionJson[mainPermission]){

                                const JSONValue = permissionJson[mainPermission][subPermission];
                                if(permissions[mainPermission][subPermission]){
                                    var status = "1";
                                }
                                else{
                                    var status = "0";
                                }

                                if(bitArr[JSONtitle] == undefined || bitArr[JSONtitle][JSONValue] == undefined)
                                {
                                    var tempArr = {};

                                    if(bitArr[JSONtitle] != undefined){
                                        tempArr = bitArr[JSONtitle];
                                    }
                                    
                                    tempArr[JSONValue] = status;
                                    bitArr[JSONtitle] = tempArr;
                                }
                                else{
                                    if(bitArr[JSONtitle][JSONValue] === "0"){

                                        bitArr[JSONtitle][JSONValue] = status;
                                    }
                                }
                            }   
                            else{
                                return {status: true, result:"fail"};
                            }
                        })
                    )
                }
                else{
                    return {status: true, result:"fail"};
                }
            })
        )

        // ===== set permission in proper format ie.{case:0101, roles: 0110} =====
        await Promise.all(
            Object.keys(bitArr).map(async(prmsion)=>{
                var bit = Object.values(bitArr[prmsion]).reverse().join('');
                
                permissionBit[prmsion] = bit;						
            })
        )
 
        return {status: true, result:permissionBit};
    }
    catch(err){
        console.log("err", err);
        return {status: false};
    }
}

module.exports= {PermissionToBinary};