const Reminder = require('../Model/Reminder');
/**
 * @description This is used for get list of All reminders
 * @param {*} query 
 * @returns Object { status: true, reminders: Array }
 * - { status: false, error: string }
 */
module.exports.getAllReminder = async (query = {}, limit, startIndex) => {

    return Reminder.find(query).skip(startIndex).limit(limit)
        .then((Reminders) => {
            return { status: true, reminders: Reminders };
        }).catch((e) => {
            return { status: false, error: e };
        });

};
/**
 * @description This is used for get count of documents 
 * @param {*} query 
 * @returns Object { status: true, reminders: Object }
 *  { status: false, error: String }
 */
 module.exports.getCount = async (query) => {
    // console.log(" Query => ", query)
    return Reminder.countDocuments(query).then((Reminders) => {
        return { status: true, reminders: Reminders };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get one Reminder By Id
 * @param {string} id 
 * @returns  Object { status: true, reminders: Object }
 * - { status: false, error: string }
 */
module.exports.getReminder = async (id) => {

    return Reminder.findById(id).then((Reminder) => {
        return { status: true, reminders: Reminder };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get one Reminder
 * @param {*} query 
 * @returns Object { status: true, reminders: Object }
 * - { status: false, error: string }
 */
module.exports.findReminder = async (query = {}) => {

    // console.log(query);
    return Reminder.findOne(query).then((Reminder) => {
        return { status: true, reminders: Reminder };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for create Reminder
 * @param {Object} ReminderData
 * @returns Object { status: true, reminders: Object }
 * - { status: false, error: string } 
 */
module.exports.createReminder = async (ReminderData) => {

    return Reminder.create(ReminderData)
        .then((result) => {
            // console.log("create Reminder Service")
            return { status: true, reminders: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};
/**
 * @description This is used for update Reminder
 * @param {String} id 
 * @param {Object} ReminderData 
 * @returns Object { status: true, reminders: Object }
 * - { status: false, error: string } 
 */
module.exports.updateReminder = async (id, ReminderData) => {

    return Reminder.findOneAndUpdate({ _id: id }, ReminderData,{new: true})
        .then((result) => {
            return { status: true, reminders: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};