const Role = require('../Model/Role');
/**
 * @description This is used for get list of All Roles
 * @param {*} query 
 * @returns Object { status: true, roles: Array }
 * - { status: false, error: string }
 */
module.exports.getAllRoles = async (query = {})=> {
    return Role.find(query).then((roles) => {
        // console.log("query :", query);
        return { status: true, roles: roles };
    }).catch((e) => {
        return { status: false, error: e };
    });
};
/**
 * @description This is used for get List By Role 
 * @param {*} query 
 * @returns Object { status: true, roles: Array }
 * - { status: false, error: string }
 */
module.exports.findByRole = async (query = {}) => {
    // console.log(query)
    return Role.find(query).then((roles) => {
        // console.log("query :", query);
        return { status: true, roles: roles };
    }).catch((e) => {
        console.log(e)
        return { status: false, error: e };
    });
};
/**
 * @description This is used for get One Role By RoleName
 * @param {Object} query 
 * @returns Object { status: true, roles: Object }
 * - { status: false, error: string }
 */
module.exports.findByRoleName = async (query = {}) => {
    // console.log(query)
    return Role.findOne(query).then((roles) => {
        // console.log("query :", query);
        return { status: true, roles: roles };
    }).catch((e) => {
        console.log(e)
        return { status: false, error: e };
    });
};
/**
 * @description This is used for get One Role By Id
 * @param {String} id 
 * @returns Object { status: true, roles: Object }
 * - { status: false, error: string }
 */
module.exports.getRole = async (id) => {

    return Role.findById(id).then((role) => {
        return { status: true, roles: role };
    }).catch((e) => {
        return { status: false, error: e };
    });
};
/**
 * @description This is used for create Role
 * @param {Object} roleData 
 * @returns Object { status: true, roles: Object }
 * - { status: false, error: string }
 */
module.exports.createRole = async (roleData) => {

    return Role.create(roleData)
        .then((role) => {
            // console.log("added Branch" + result)
            return { status: true, roles: role }
        })
        .catch((err) => {
            console.log(err)
            return { status: false, err }
        })
};
/**
 * @description This is used for updates Role
 * @param {String} id 
 * @param {Object} data 
 * @returns Object { status: true, roles: Object }
 * - { status: false, error: string }
 */
module.exports.editRole = async (id, data) => {
    return Role.findOneAndUpdate({ _id: id }, data, { new: true })
        .then((role) => {
            // console.log("update Branch",branch)
            return { status: true, roles: role }
        })
        .catch((err) => {
            console.log(err)
            return { status: false, err }
        })
};

module.exports.deleteRole = async (req, res) => {
    // return res.status(200).send({ message: "Hi There Its  Admin" });
};



