const Subscription = require('../Model/Subscriptions');
/**
 * @description This is used for get list of All Subscriptions 
 * @param {*} query 
 * @returns Object { status: true, subscriptions: Array }
 * - { status: false, error: String }
 */
module.exports.findAll = async (query, { display = [], sort = { createdAt: -1 }, populate = [] }, startIndex, limit) => {

    return Subscription.find(query, display).populate(populate).skip(startIndex).sort(sort).limit(limit)
        .then((Subscriptions) => {
            return { status: true, subscriptions: Subscriptions };
        }).catch((e) => {
            return { status: false, error: e };
        });
};

/**
 * @description This is used for get One Subscription By Id
 * @param {String} id 
 * @returns Object { status: true, subscriptions: Object }
 * - { status: false, error: String }
 */
module.exports.findById = async (id) => {
    return Subscription.findById(id).then((Subscription) => {
        return { status: true, subscriptions: Subscription };
    }).catch((e) => {
        return { status: false, error: e };
    });
};

/**
 * @description This is used for create Subscription
 * @param {Object} SubscriptionData 
 * @returns Object { status: true, subscriptions: Object }
 * - { status: false, error: String }
 */
module.exports.create = async (SubscriptionData) => {
    return Subscription.create(SubscriptionData)
        .then((result) => {
            return { status: true, subscriptions: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })
};

/**
 * @description This is used for update Subscription
 * @param {String} id 
 * @param {Object} SubscriptionData 
 * @returns Object { status: true, subscriptions: Object }
 * - { status: false, error: String }
 */
module.exports.update = async (id, SubscriptionData) => {
    return Subscription.findOneAndUpdate({ _id: id }, SubscriptionData)
        .then((result) => {
            return { status: true, subscriptions: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })
};