const { zone } = require('../Constants/permission');
const Zone = require('../Model/Zone');
/**
 * @description This is used for get list of All Zones
 * @param {*} query 
 * @returns Object { status: true, zones: Array }
 * - { status: false, error: string }
 */
module.exports.getAllZones = async (query = {}, limit, startIndex) => {
    // console.log("query : ",JSON.stringify(query));
    return Zone.find(query).collation({ 'locale': 'en' }).sort({ zoneName: 1 }).skip(startIndex).limit(limit)
        .then((Zones) => {
            return { status: true, zones: Zones };
        }).catch((e) => {
            return { status: false, error: e };
        });

};
/**
 * @description This is used for get count of documents 
 * @param {*} query 
 * @returns Object { status: true, zones: Object }
 *  { status: false, error: String }
 */
module.exports.getCount = async (query) => {
    // console.log(" Query => ", query)
    return Zone.countDocuments(query).then((Zone) => {
        return { status: true, zones: Zone };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get One zone By Id 
 * @param {String} id 
 * @returns Object { status: true, zones: Object }
 * - { status: false, error: string }
 */
module.exports.getZone = async (id) => {

    return Zone.findById(id).then((Zone) => {
        return { status: true, zones: Zone };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get One zone 
 * @param {*} query 
 * @returns Object { status: true, zones: Object }
 * - { status: false, error: string }
 */
module.exports.getOneZone = async (query = {}) => {

    return Zone.findOne(query)
        .then((Zone) => {
            return { status: true, zones: Zone };
        }).catch((e) => {
            return { status: false, error: e };
        });

};
/**
 * @description This is used for  create Zone 
 * @param {*} ZoneData 
 * @returns  Object { status: true, zones: Object }
 * - { status: false, error: string }
 */
module.exports.createZone = async (ZoneData) => {

    return Zone.create(ZoneData)
        .then((result) => {
            // console.log("create Zone Service")
            return { status: true, zones: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })
};
/**
 * @description  This is used for update Zone 
 * @param {String} id 
 * @param {Object} ZoneData 
 * @returns  Object { status: true, zones: Object }
 * - { status: false, error: string }
 */
module.exports.updateZone = async (id, ZoneData) => {

    return Zone.findOneAndUpdate({ _id: id }, ZoneData, { new: true })
        .then((result) => {
            return { status: true, zones: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};
/**
 * @description This is used for update zone from query
 * @param {Object} find 
 * @param {Object} ZoneData 
 * @returns Object { status: true, zones: Object }
 * - { status: false, error: string }
 */
module.exports.findUpdateZone = async (find, ZoneData) => {

    return Zone.findOneAndUpdate(find, ZoneData, { new: true })
        .then((result) => {
            return { status: true, zones: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};
/**
 * @description This is used for update Many zones
 * @param {Object} find_query 
 * @param {Object}  ZoneData
 * @returns Object { status: true, zones: result }
 * -  { status: false, err }
 */
module.exports.updateMultipleZone = async (id, update, filter) => {
    // console.log(id, " ===",update,"====" , filter);
    return Zone.updateMany(id, update, filter)
        .then((result) => {
            return { status: true, zones: result }
        })
        .catch((err) => {
            console.log(err)
            return { status: false, err }
        })
};
/**
 * @description  This is used for delete Zone 
 * @param {String} id 
 * @returns  Object { status: true, zones: Object }
 * - { status: false, error: string }
 */
module.exports.deleteZone = async (id) => {

    return Zone.remove({ _id: id })
        .then((result) => {
            return { status: true, zones: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};
