const express = require('express');
const router = express.Router();
const ClientController = require('../Controllers/Client.controller');
const isAuth = require('../Middleware/isAuth');
const pagination = require('../Middleware/pagination');
const isAdmin = require('../Middleware/isAdmin');
const isPermission = require('../Middleware/isPermission');
const validation = require('../Middleware/validation');
const uploadImage = require('../Middleware/uploadFile');

// For list all Clients 
router.get('/Clients', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "client", value: "list" }) }, pagination, ClientController.getAllClients);

// For get one client 
router.get('/Clients/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "client", value: "view" }) }, ClientController.getClient);

// For create new Client
router.post('/Clients', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "client", value: "create" }) }, uploadImage.logo, validation.clientValidation, ClientController.createClient);

// For update Client 
router.put('/Clients/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "client", value: "edit" }) }, uploadImage.logo, validation.clientValidation, ClientController.updateClient);

// For list all Branches Client wise 
router.get('/Clients/:id/branches', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "branch", value: "list" }) }, ClientController.getClientBranches);

// For list all zones Client wise 
router.get('/Clients/:id/zones', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "branch", value: "list" }) }, ClientController.getClientZones);

// router.post('/Clients/:id/updatejson', isAuth, isAdmin, ClientController.updateJson);

// router.delete('/Clients/:id', isAuth, isAdmin, ClientController.deleteClient);

module.exports = router;

