const express = require('express');
const router = express.Router();
const CreateContact = require('../Controllers/CreateContact.controller');
const isAuth = require('../Middleware/isAuth');
const isPermission = require('../Middleware/isPermission');
const validation = require('../Middleware/validation');
const pagination = require('../Middleware/pagination');
const uploadFile = require('../Middleware/uploadFile');

// For get createContacts list 
router.get('/createContacts', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "createContact", value: "list" }) }, pagination, CreateContact.getAllCreateContacts);

// For get one createContact
router.get('/createContacts/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "createContact", value: "view" }) }, CreateContact.getCreateContact);

// For create CreateContact i.e. created from APP
router.post('/createContacts', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "createContact", value: "create" }) }, uploadFile.uploadImages, validation.createContactValidation, CreateContact.createCreateContact);

// For update CreateContact 
router.put('/createContacts/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "createContact", value: "edit" }) }, uploadFile.uploadImages, validation.createContactValidation, CreateContact.updateCreateContact);

// For update CreateContact Status i.e. change from Web 
router.post('/createContacts/updateStatus/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "createContact", value: "edit" }) }, CreateContact.updateStatus);


router.delete('/createContact/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "createContact", value: "delete" }) }, CreateContact.deleteContact);

module.exports = router;

