const express = require('express');
const router = express.Router();
const UserController = require('../Controllers/User.controller');
const CreditOfficerController = require('../Controllers/CreditOfficer.controller');
const isAuth = require('../Middleware/isAuth');
const isAdmin = require('../Middleware/isAdmin');
const isPermission = require('../Middleware/isPermission');
const validation = require('../Middleware/validation');
const pagination = require('../Middleware/pagination');
const uploadImage = require('../Middleware/uploadFile');

// For get list of all creditOfficer with its case Count
router.get('/creditOfficers', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "list" }) }, pagination, CreditOfficerController.getAllCreditOfficer);

// For get one User
router.get('/creditOfficers/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "view" }) }, CreditOfficerController.getCreditOfficer);

// For create User
router.post('/creditOfficers', isAuth, isAdmin, uploadImage.userProfile, validation.createUserValidation, CreditOfficerController.createCreditOfficer);

// For update User
router.put('/creditOfficers/:id', isAuth, isAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "edit" }) }, uploadImage.userProfile, validation.createUserValidation, CreditOfficerController.updateCreditOfficer);

// For delete creditOfficer 
router.delete('/creditOfficers/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "delete" }) }, CreditOfficerController.deleteCreditOfficer);

router.get('/CreditOfficer-BranchHistory/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "list" }) }, CreditOfficerController.getCreditOfficerBranchHistory)

module.exports = router;

