const express = require('express');
const router = express.Router();
const TemplateController = require('../Controllers/ReportTemplate.controller');
const isAuth = require('../Middleware/isAuth');
const pagination = require('../Middleware/pagination');
const isAdmin = require('../Middleware/isAdmin');
const isSuperAdmin = require('../Middleware/isSuperAdmin');
const isPermission = require('../Middleware/isPermission');
const validation = require('../Middleware/validation');

// For get list of all Report Templates 
router.get('/reportTemplatesMaster', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "template", value: "list" }) }, pagination, TemplateController.getAllTemplateMaster);
router.get('/reportTemplates', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "template", value: "list" }) }, pagination, TemplateController.getAllReportTemplates);

// For get only one Report Template
router.get('/reportTemplates/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "template", value: "view" }) }, TemplateController.getReportTemplate);

// For create Report Template 
router.post('/reportTemplates', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "template", value: "create" }) }, validation.reportTemplateValidation, TemplateController.createReportTemplate);

// For update Report Template
router.put('/reportTemplates/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "template", value: "edit" }) }, validation.reportTemplateValidation, TemplateController.updateReportTemplate);

// For check Already Exist Template or not 
router.post('/reportTemplates/checkAlreadyExistTemplate', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "template", value: "create" }) }, TemplateController.checkAlreadyExistTemplate);

module.exports = router;