const Router = require('express').Router;
const glob = require('fast-glob');
const fs = require('fs');
const path = require('path');
const { logger } = require('../logger');

const BASE_DIR = path.join(__dirname, '..');

module.exports = async function RouteLoader(globPattern) {
    let router = Router();
    let files = [];
    try {
        files = await glob(globPattern, { cwd: BASE_DIR });
    } catch (error) {
        console.error(error);
    }
    console.group('Routes Loading...');
    for (const file of files) {
        if (fs.statSync(file).isFile() && path.extname(file).toLowerCase() === '.js') {
            try {
                const routeModule = await require(path.resolve(file));
                router.use(routeModule);
                console.log(`=> Route : ${file} Loaded! `)
            } catch (e) {
                logger.error(`Error when loading route file: ${file} [ ${e.toString()} ]`);
            }
        }
    }
    console.groupEnd("-------------------------\n");

    return router;
}